#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <time.h>

#include "v_inp.h"
#include "u_plot.h"
#include "v_plotstr.h"
#include "xmosplo.h"


 typedef struct {
          short int x, y;
       } XPoint;
#include "u_pl_loc.h"
#include "u_plot1.h"
char  FontName[5][80];


#define FRAME 5      /* pixel frame */
#define TICS 11
#define TICLEN 3
#define TEXTHEIGHT 15       /* pixel */
#define TOP_FRAME 0.94
#define BOTTOM_FRAME 0.15
#define LEFT_FRAME 0.12
#define RIGHT_FRAME 0.95
#define INFO_POSX ((RIGHT_FRAME+LEFT_FRAME)/2)
#define TITLE_POSY (BOTTOM_FRAME*0.1)
#define XLABEL_POSY (BOTTOM_FRAME*0.2)
#define YLABEL_POSY (TOP_FRAME*0.9)
#define Y2LABEL_POSY (TOP_FRAME*0.85)

#define VHIGH 1.0e38
#define DEFAULTFORMATX "%6.2f"
#define DEFAULTFORMATY "%10.1f"

static int xoffset=0, numpl=1 ,numplot=0;
static int yoffset=0;static float zoomf=1.0;
static int status, notitle=0;
static int symbols_loaded = 0;
static int size=0;

char plotmessages[14][40] = {"ok",
                            "No more handles",
                            "Out of memory",
                            "No such handle",
                            "No more handles of this type",
                            "Can't open Display",
                            "No such device",
                            "Error opening outputfile",
                            "file MODEL.SYM not found",
                            "file PSPLOT_HEADER not found",
			    "file PLOT.FONT not found",
			    "files MODEL.SYM and PLOT.FONT not found",
                            " exit"};

/*=========== Init/Exit plot package ===============*/

#define SYMFILE "MODEL.SYM"
#define SYMENV  "EFFIUTPATH"
#define MAXSYMBOLS 34
#define help help_


char wdescription[80]="EFFI plotwindow";

static struct plotstruct plc[MAXHANDLES];
static int is_window_device;
static int iframe=3;

static handleptr handles[MAXHANDLES];
static float size_factor; 

plotstruct *get_plotconfig(){
   return(&plc[MAXHANDLES]);
}


static int  frame, ticlen, sizex, sizey;
static int a_h = -1;
static int mhandle =0;
static int do_auto_update = FALSE;

static int wksp[MAXHANDLES+1];
static int n_sub, n_data, n_theo;

static int have_subspectra, plot_subspectra, pl_subsp[MAXSUB],
           have_theo,plot_theo,have_data,plot_data,have_residue, plot_auto,
           winsizex, winsizey,winposx,winposy;

static float *pxd,*pyd,*pyderr,*pyres,*pxt,*pyt, *psub[8], *p6th, *p9th;
static int *pfeder;
#define FONTSFILE "PLOT.FONT"
#define SYMENV  "EFFIUTPATH"
static int fonts_loaded = 0;


/*=====================================================================*/
/* Universal plot routines for various devices                         */
/* V2.0                    by A.Vef                           08.12.93 */
/*=====================================================================*/
/* How it works:                                                       */
/*  - this module provides device-independent routines. A call to      */
/*    plot_open() connects them to the device-specific implementation  */
/*  - Method: pointers to the functions                                */
/* How to add a new driver:                                            */
/*  - define an appropriate constant PLOT_DEV_xxx in U_PL_DEF.H        */
/*  - write the driver according to the examples.                      */
/*     = all routines should be named plotd_<name>_<devtype>(),        */
/*       e.g. plotd_clear_tek()                                        */
/*     = you have to provide ALL routines, even if they are meaning-   */
/*       less for this device (e.g. input on plotter), except the      */
/*       following:                                                    */
/*     = Optional Routines:                                            */
/*         plotd_getpointer_...()                                      */
/*         plotd_checkevent_...()                                      */
/*       their presence is checked at runtime, if they do not exist,   */
/*       the calls are ignored.                                        */
/*         plotd_polysymbol_...()                                      */
/*       checked at runtime, if not present, symbols are plotted using */
/*       plotd_line_...().                                             */
/*  - add the call to plotd_init_<devtype>() to plot_open()            */
/*    if the device is limited to a specific environment, use "#ifdef" */
/*=====================================================================*/


struct symboltyp symbols[MAXSYMBOLS];

void help_(char []);

void load_font(){
  int i,j,k;
   char *tmp,fname[80],instr[80];
   float out[16];
   FILE *fontfile;

   if (fonts_loaded) return;
   if ((tmp=getenv(SYMENV))!=NULL){ strcpy(fname,tmp);strcat(fname,"/");
   strcat(fname,FONTSFILE);}
   else strcpy(fname,FONTSFILE);
   /*   printf("load_sym3 %s\n",fname);*/
   if ((fontfile=fopen(fname,"r"))==NULL) return;
   for (i=0;i<5;i++){
    fgets(instr,80,fontfile);
    k=strlen(instr); instr[k-1]=instr[k];
    strcpy(FontName[i],instr);
    /*printf("bl[0]=%d i=%d l=%d ,%s \n",FontName[i][k-1],i,k,FontName[i]);*/
   }
   fclose(fontfile);
   fonts_loaded = TRUE;
}


void load_sym(){
   int i,j;
   char *tmp,fname[80],instr[80];
   float out[16];
   FILE *symfile;

   if (symbols_loaded) return;
   if ((tmp=getenv(SYMENV))!=NULL){ strcpy(fname,tmp);strcat(fname,"/");
   strcat(fname,SYMFILE);}
   else strcpy(fname,SYMFILE);
   /*   printf("load_sym3 %s\n",fname);*/
   if ((symfile=fopen(fname,"r"))==NULL) return;
   for (i=1;i<MAXSYMBOLS;i++){
      fgets(instr,70,symfile);
      trans(instr,out); symbols[i].vec=out[0];
      symbols[i].x1 = malloc(symbols[i].vec*sizeof(symbolcoordtyp));
      symbols[i].x2 = malloc(symbols[i].vec*sizeof(symbolcoordtyp));
      symbols[i].y1 = malloc(symbols[i].vec*sizeof(symbolcoordtyp));
      symbols[i].y2 = malloc(symbols[i].vec*sizeof(symbolcoordtyp));
      for (j=0; j<symbols[i].vec; j++)
	{ fgets(instr,70,symfile);
          trans(instr,out);
	  symbols[i].x1[j] = out[0];
	  symbols[i].y1[j] = out[1];
	  symbols[i].x2[j] = out[2];
	  symbols[i].y2[j] = out[3];
	}
   }
   fclose(symfile);
   symbols_loaded = TRUE;
}


int plot_init(){
   int i;
   for (i=0; i<MAXHANDLES; i++) handles[i] = NULL;
   load_sym();
   load_font();
   if (symbols_loaded) status = PLOT_STAT_OK;
                 else  status = PLOT_STAT_NOSYMBOLS;
   if (fonts_loaded) status = PLOT_STAT_OK;
                 else  status = PLOT_STAT_NOFONTS;
   return(symbols_loaded);
}

void plotexita(){
   int i;
   for (i=0; i<MAXHANDLES; i++) if (handles[i]){
    plot_close(i); handles[i] = 0; 
			wptr.intptr[i]=0;}
}

void plotexit(int i){
   if (handles[i]){
        plot_close(i); handles[i] = 0;
        wptr.intptr[i]=0;}
}


void calculate_transformation(int handle){
   handleptr h;
   h = handles[handle];
#define scale_one(x1, x2, i1, i2, f, o){\
   int i; float d;\
   i = i2 - i1;     d = x2 - x1;\
   f = d/i;         o = x1 - f * i1;}
   scale_one(h->xwmin,h->xwmax,h->xvmin,h->xvmax,h->xfact,h->xoffs);
   scale_one(h->ywmin,h->ywmax,h->yvmin,h->yvmax,h->yfact,h->yoffs);
/*printf(" handle=%ih->xwmin= %f h->xwmax= %f \n",handle,h->xwmin,h->xwmax);*/
#undef scale_one
}


/* convert screen to world coordinates */
void screen_to_world(int handle, int xi, int yi, float *xr, float *yr)
{ /*  handleptr h;
   char c;
   h = handles[handle];    printf("2handle=%i \n",handle);*/
   *xr = handles[handle]->xfact * xi + handles[handle]->xoffs;
   *yr = handles[handle]->yfact * yi + handles[handle]->yoffs;
}
/* convert world to screen coordinates */
void world_to_screen(int handle, float xr, float yr, int *xi, int *yi)
{
   *xi = (xr - handles[handle]->xoffs) / handles[handle]->xfact;
   *yi = (yr - handles[handle]->yoffs) / handles[handle]->yfact;
/*printf(" xi=%i xr=%f h->xoffs= %f \n",*xi,xr,handles[handle]->xoffs);*/
}


/* Open a plot handle, i.e. Window, Plotterfile,... */
/* Returns -1 on error, otherwise handle             */
/* Name is Windowtitle or filename for Plotter      */
int plot_open(int handle,int devicetyp, 
    int sizex, int sizey, int posx, int posy,char name[],char *backgroundcolor)
{int i;

   if (devicetyp==PLOT_DEV_DEFAULT) devicetyp = PLOT_DEV_NO;
   if ((devicetyp<=PLOT_DEV_NO) || (devicetyp>=PLOT_DEV_LASTDEV)) return(-1);

   for(i=0;i<MAXHANDLES;i++){handles[i] = (handleptr)wptr.intptr[i];}

   /*handle = 0;*/
   while ((handles[handle]) && (handle <MAXHANDLES)) handle++;
   if (handle==MAXHANDLES){ status = PLOT_STAT_NOMOREHANDLES; return(-1);}
   handles[handle] = malloc(sizeof(handletyp));
   if (handles[handle]==NULL){ status = PLOT_STAT_NOMEMORY; return(-1);}
   handles[handle]->devtype = devicetyp;
   /* Initialize optional capabilities to NULL, so we can check */
   /* if they are supported.                                    */
   handles[handle]->ploth_checkevent=NULL;
   handles[handle]->ploth_getpointer=NULL;
   handles[handle]->ploth_polysymbol=NULL;
   handles[handle]->ploth_rectangle=NULL;
   handles[handle]->capabilities=PLOT_CAPABILITY_NONE;		 
   status = plotd_init_x11(handle,handles[handle],
                    sizex,sizey,posx,posy,name,backgroundcolor); 
   if (status==PLOT_STAT_OK){
      (*handles[handle]->ploth_getdevice)(handle,
				       &handles[handle]->xposw,
				       &handles[handle]->yposw,
				       &handles[handle]->xvmax,
				       &handles[handle]->yvmax);

      handles[handle]->xwmin=0;      handles[handle]->xwmax=1;
      handles[handle]->ywmin=0;      handles[handle]->ywmax=1;
      handles[handle]->xvmin=0;
      handles[handle]->yvmin=0;
      handles[handle]->akt_x=0;      handles[handle]->akt_y=1;
      calculate_transformation(handle);
      handles[handle]->clip=FALSE;
      handles[handle]->symbol_factor = 1;

      return(handle);
   }else{
      free(handles[handle]); handles[handle] = NULL; return(-1);
   }
}

/* return to textmode:                         */
/*  - stdin/stdout available for text output   */
/*  - graphic output prohibited                */
void plot_textmode(){
   int i;
   for (i=0; i<MAXHANDLES; i++)
     if (handles[i]) (*handles[i]->ploth_textmode)(i);
}

/* switch to graphic mode                                  */
/*  - inquire actual size for each open handle (XWindow !) */
/*  - allow graphic output                                 */
void plot_graphmode(){
   int i;
   for (i=0; i<MAXHANDLES; i++){
      if (handles[i]) (*handles[i]->ploth_graphmode)(i);
   }
}

/* return last error message */
char *plot_message(){
   return plotmessages[status];
}


/*===================== Handle =====================*/
/* Modify existing handle, returns status */
int plot_modify(int handle, int sizex, int sizey, int posx, int posy,
   	        char name[]){
   if (! handles[handle]){status=PLOT_STAT_NOSUCHHANDLE; return(FALSE);}
   status = (*handles[handle]->ploth_modify)(handle,sizex,sizey,posx,posy,name);
   return (status==PLOT_STAT_OK);
}


/* close a handle */
void plot_close(int handle){
   if (! handles[handle]) return;
   (*handles[handle]->ploth_close)(handle);
   free(handles[handle]); handles[handle] = 0;
}

void file_close(int handle){
   if (! handles[handle]) return;
   (*handles[handle]->fileh_close)(handle);
}

/* Clear a handle: Clear Screen or Window, FormFeed on Plotter */
void plot_clear(int handle){
   if (! handles[handle]) return;
   (*handles[handle]->ploth_clear)(handle);
}


/*===================== Inquire Status ========================*/
/* Status of handle, or general status if handle==0 */
int plot_status(int handle){
   if (handle==0) return(status);
   if (handles[handle]) return(PLOT_STAT_OK);
   return(PLOT_STAT_NOSUCHHANDLE);
}

/* return devicetype and dimensions */
int plot_getdev(int handle,int *posx,int *posy,int *sizex,int *sizey){
   if (! handles[handle]) { status = PLOT_STAT_NOSUCHHANDLE;
      return(PLOT_DEV_NO);   }
   (*handles[handle]->ploth_getdevice)(handle,posx,posy,sizex,sizey);

   return( handles[handle]->devtype);
}

unsigned long plot_capabilities(int handle, unsigned long capability){
   if (capability==PLOT_CAPABILITY_ALL) 
      return handles[handle]->capabilities;
   return ((handles[handle]->capabilities & capability) != 0);
}
   
/*====================== Modify Parameters ====================*/
/* All routines return the value actually used, which may be   */
/* different from the value desired                            */

int plot_setfgbyname(int handle, char *cforeground){
  return((*handles[handle]->ploth_setfgbyname)(handle,cforeground));}

/* Set writing mode (replace/xor/or/erase/...) */
int plot_setmode(int handle, int mode){
  return((*handles[handle]->ploth_setmode)(handle,mode));}

/* Set data window */
void plot_setwin(int handle, float xmin, float ymin,
		             float xmax, float ymax){
   handles[handle]->xwmin = xmin;   handles[handle]->xwmax = xmax;
   handles[handle]->ywmin = ymin;   handles[handle]->ywmax = ymax;
   calculate_transformation(handle);
}

/* Set data viewport */
void plot_setview(int handle, int xmin, int ymin, int xmax, int ymax){
   handles[handle]->xvmin = xmin;   handles[handle]->xvmax = xmax;
   handles[handle]->yvmin = ymin;   handles[handle]->yvmax = ymax;
   calculate_transformation(handle);
}

/* set clipping */
void plot_setclip(int handle, int clip){
   handles[handle]->clip = clip;
}

/* set symbol size */
float plot_setsymsize(int handle, float size){
   handles[handle]->symbol_factor = size;
   return(handles[handle]->symbol_factor);
}


/*====================== inquire Parameters ====================*/
/* get foreground colour index */
int plot_getfg(int handle){
   return((*handles[handle]->ploth_getfg)(handle));
}


/* get background colour index */
int plot_getbg(int handle){
   return((*handles[handle]->ploth_getbg)(handle));
}


/* get writing mode (replace/xor/or/erase/...) */
int plot_getmode(int handle){
   return((*handles[handle]->ploth_getmode)(handle));
}


/* get data window */
void plot_getwin(int handle, float *xmin, float *xmax,
		             float *ymin, float *ymax){
    *xmin = handles[handle]->xwmin;    *xmax = handles[handle]->xwmax;
    *ymin = handles[handle]->ywmin;    *ymax = handles[handle]->ywmax;
}

/* get data viewport */
void plot_getview(int handle, int *xmin, int *xmax, int *ymin, int *ymax){
    *xmin = handles[handle]->xvmin;    *xmax = handles[handle]->xvmax;
    *ymin = handles[handle]->yvmin;    *ymax = handles[handle]->yvmax;
}


/* get clipping */
int plot_getclip(int handle){return(handles[handle]->clip);
}


/* get symbol size */
float plot_getsymsize(int handle){return(handles[handle]->symbol_factor);
}


/*=========== Output routines, world coordinates only ============*/
int in_range(int handle, float x, float y)
{ if (!handles[handle]->clip) return(1);
  if (x > handles[handle]->xwmax) return(0);
  if (y > handles[handle]->ywmax) return(0);
  if (x < handles[handle]->xwmin) return(0);
  if (y < handles[handle]->ywmin) return(0);
  return(1);
}

void plot_point(int handle, float X, float Y){
   int IX, IY;
   if (in_range(handle,X,Y)){
      world_to_screen(handle,X,Y,&IX,&IY);
      (*handles[handle]->ploth_point)(handle,X,Y);
}}

void plot_line(int handle, float X0, float Y0, float X1, float Y1){
   int IX0, IY0, IX1, IY1;
   if ((in_range(handle,X0,Y0)) && (in_range(handle,X1,Y1))){
      world_to_screen(handle,X0,Y0,&IX0,&IY0);
      world_to_screen(handle,X1,Y1,&IX1,&IY1);
      (*handles[handle]->ploth_line)(handle,IX0,IY0,IX1,IY1);
}}
void plot_linevc(int handle,   short int IX0, short int IY0, 
                               short int IX1, short int IY1){
      (*handles[handle]->ploth_linevc)(handle,IX0,IY0,IX1,IY1);
}


/*=================== Text output ================*/
/* set text alignment */
void plot_settextalign(int handle, int xalign, int yalign){
   (*handles[handle]->ploth_settextalign)(handle,xalign,yalign);
}


/* write txt at position (X,Y) */
void plot_writetext(int handle, int worldcoor, float X, float Y, char txt[]){
   int IX,IY,ixs,iys;
   if (worldcoor) world_to_screen(handle,X,Y,&IX,&IY);
   else { IX = X; IY = Y; }
   (*handles[handle]->ploth_writetext)(handle,IX,IY,txt,&ixs,&iys);
   (*handles[handle]->ploth_drawstring)(handle,ixs,iys,txt);
}

/* write a character at position (X,Y) */
void plot_writechar(int handle, int worldcoor, float X, float Y, char *ch){
   int IX,IY;
   if (worldcoor) world_to_screen(handle,X,Y,&IX,&IY);
   else { IX = X; IY = Y; }
   (*handles[handle]->ploth_writechar)(handle,IX,IY,ch);
}

/* erase txt at position (X,Y). Usually done by plot_setmode(ERASE) and */
/* plot_writetext, but in some cases it has to be done in another way...*/
void plot_erasetext(int handle, int worldcoor, float X, float Y, char txt[]){
   int IX,IY;
   if (worldcoor) world_to_screen(handle,X,Y,&IX,&IY);
   else { IX = X; IY = Y; }
   (*handles[handle]->ploth_erasetext)(handle,IX,IY,txt);
}


/*================= Pointer position ==============*/

/* Get pointer position and return terminator. Returns 0 if handle if */
/* output only (Plotter,...), character if a key was pressedm, or one */
/* of the codes defined as PLOT_TRM_... in "u_pl_def.h" 
   plotd_getpointer_X11(&handle is a direct access to X11 because
   the handle is is an output of this routine              */
int plot_getpointer(int ignore, float *X, float *Y){
   int IX, IY, trm, handle;
   trm =plotd_getpointer_X11(&handle,mhandle,ignore,&IX,&IY);
   a_h=handle;
  if (trm != PLOT_TRM_ABORT) screen_to_world(a_h,IX,IY,X,Y);
  if(ignore == 0){
       if(plot.logx)*X=exp(*X*2.3025851);
       if(plot.logf)*Y=exp(*Y*2.3025851);}
   return(trm);
}

int plot_getpointer2d(int ignore, float *X, float *Y){
   int trm, handle, IX, IY;
   float h,r;
   
   trm =plotd_getpointer_X11(&handle,mhandle,ignore,&IX,&IY);   a_h=handle;
  if(ignore == 0){
   h=cmpixel*zoomf*sizex/stwinsizex;
   r=((float)sizex*(1.0+zoomf)/2.+(float)xoffset*zoomf*sizex/stwinsizex-IX)/h;
       *X=plot.xyzdz[4]+plot.xyzdz[5]/pixwin_cm*r;
       if(plot.logx)*X=exp(*X*2.3025851);
       /* printf("xyzdz[4]=%f xyzdz[5]=%f r=%f   xoffset=%d \n",
	  plot.xyzdz[4],plot.xyzdz[5],r,xoffset);*/
   h=cmpixel*zoomf*sizey/stwinsizey;
   r=((float)sizey*(1.0+zoomf)/2.-(float)yoffset*zoomf*sizey/stwinsizey-abs(IY-sizey))/h;
       *Y=plot.xyzdz[6]+plot.xyzdz[7]/pixwin_cm*r;
       if(plot.logf)*Y=exp(*Y*2.3025851);
   /*printf("xyzdz[6]=%f xyzdz[7]=%f  r=%f\n\n", plot.xyzdz[6],plot.xyzdz[7],r);*/
  }
   return(trm);
}
int plot_getpointer3d(int ignore,int *IX, int *IY){
   int trm, handle;
   trm =plotd_getpointer_X11(&handle,mhandle,ignore,IX,IY);   a_h=handle;
   return(trm);
}

/* Check for event, if event present, get pointer position and return */
/* terminator. Returns 0 if handle if output only (Plotter,...) or    */
/* no event, character if a key was pressedm, or one of the codes     */
/* defined as PLOT_TRM_... in "u_pl_def.h"                            */
int plot_checkevent(int handle, float *X, float *Y){
   int IX, IY, trm;
   if (handles[handle]->ploth_checkevent==NULL) return 0;
   trm = (*handles[handle]->ploth_checkevent)(handle,&IX,&IY);

   if (trm != PLOT_TRM_ABORT) screen_to_world(handle,IX,IY,X,Y);
   return(trm);
}

/*==================== Future work ================*/
/* Set font for text in graphic mode */
/* Define color                      */


void plot_lineabs(int handle, float x1, float y1, float x2, float y2){
   plot_line(handle,x1,y1,x2,y2);
   handles[handle]->akt_x = x2; handles[handle]->akt_y = y2;
}

void plot_rectangle(int handle, float x1, float y1, float x2, float y2,
                    int filled){
   if (handles[handle]->ploth_rectangle==NULL){
      plot_lineabs(handle,x1,y1,x2,y1); plot_lineto(handle,x2,y2);
      plot_lineto(handle,x1,y2); plot_lineto(handle,x1,y1);
   } else 
     handles[handle]->ploth_rectangle(handle, x1, y1, x2, y2, filled);
}

void plot_lineto(int handle, float x, float y){
   plot_lineabs(handle,handles[handle]->akt_x,
		       handles[handle]->akt_y,x,y);
}

void plot_linerel(int handle, float x, float y){
   x = handles[handle]->akt_x + x; y = handles[handle]->akt_y + y;
   plot_lineto(handle,x,y);
}

void plot_moverel(int handle, float x, float y){
   handles[handle]->akt_x = handles[handle]->akt_x + x;
   handles[handle]->akt_y = handles[handle]->akt_y + y;
}

void plot_moveto(int handle, float x, float y){
   handles[handle]->akt_x = x;
   handles[handle]->akt_y = y;
}

void plot_polyline(int handle, int n, float x[], float y[]){
   int *ix, *iy, i, j, size;
   size = n*sizeof(int); ix = malloc(size); iy = malloc(size);
   if ((ix==NULL) || (iy==NULL)){
      free(ix); free(iy);
      status = PLOT_STAT_NOMEMORY; return;}
   j = 0;
   for (i=0; i<n; i++) {
       if (in_range(handle,x[i],y[i])){
        world_to_screen(handle,x[i],y[i],ix+j,iy+j); j++;
	} else {if (j>1) (*handles[handle]->ploth_polyline)(handle,j,ix,iy);j=0;}
    }
   if (j>1)(*handles[handle]->ploth_polyline)(handle,j,ix,iy);
   free(ix); free(iy);
}

void plot_polylinevc(int handle, int n, XPoint *x){
   (*handles[handle]->ploth_polylinevc)(handle,n,x);
} 
void plot_fillpolygon(int handle, int n, XPoint *x){
   (*handles[handle]->ploth_fillpolygon)(handle,n,x);
}

void vector_polyline(int handle, int n, float x[], float y[]){
   int *ix, *iy, i, j, size;
   size = n*sizeof(int); ix = malloc(size); iy = malloc(size);
   if ((ix==NULL) || (iy==NULL)){
      free(ix); free(iy);
      status = PLOT_STAT_NOMEMORY; return;}
   j = 0;
   for (i=0; i<n; i++) {
        world_to_screen(handle,x[i],y[i],ix+j,iy+j); j++;
    }
   if (j>1){(*handles[handle]->ploth_polyline)(handle,j,ix,iy);
       if(sizey>600){for(i=0;i<j;i++){ix[i]=ix[i]+1;iy[i]=iy[i]+1;};
            (*handles[handle]->ploth_polyline)(handle,j,ix,iy);}
           }
   free(ix); free(iy);
}



void plot_symbol(int handle, int sym, float x, float y)
{  int i, ix, iy;
   if (! in_range(handle,x,y)) return;
   world_to_screen(handle,x,y, &ix, &iy);
   if ((symbols_loaded) && (sym>0) && (sym<34)){
     for (i=0; i<symbols[sym].vec; i++)
	(*handles[handle]->ploth_line)(handle,
	   (int)(ix+symbols[sym].x1[i] * handles[handle]->symbol_factor),
	   (int)(iy+symbols[sym].y1[i] * handles[handle]->symbol_factor),
	   (int)(ix+symbols[sym].x2[i] * handles[handle]->symbol_factor),
	   (int)(iy+symbols[sym].y2[i] * handles[handle]->symbol_factor));
   } else (*handles[handle]->ploth_point)(handle,ix,iy);
}

void plot_symbolvc(int handle, int sym, short int ix, short int iy )
{  int i;
   if ((symbols_loaded) && (sym>0) && (sym<34)){
     for (i=0; i<symbols[sym].vec; i++)
	(*handles[handle]->ploth_linevc)(handle,
	   (int)(ix+symbols[sym].x1[i] * handles[handle]->symbol_factor),
	   (int)(iy+symbols[sym].y1[i] * handles[handle]->symbol_factor),
	   (int)(ix+symbols[sym].x2[i] * handles[handle]->symbol_factor),
	   (int)(iy+symbols[sym].y2[i] * handles[handle]->symbol_factor));
   } else (*handles[handle]->ploth_pointvc)(handle,ix,iy);
}

void plot_polysymbol(int handle, int pts, int sym, float X[], float Y[]){
   int i, *ix, *iy, ipts;
   if ((symbols_loaded) && (sym>0) && (sym<34)){
     if (handles[handle]->ploth_polysymbol==NULL)
       for (i=0; i<pts; i++) plot_symbol(handle,sym,X[i],Y[i]);
     else{
       ix = malloc(pts * sizeof(int));
       iy = malloc(pts * sizeof(int));
       ipts = 0;
       for (i=0; i<pts; i++) {
         if (! in_range(handle,X[i],Y[i])) continue;
         world_to_screen(handle,X[i],Y[i], &ix[ipts], &iy[ipts]);
	 ipts++;
       }
       (*handles[handle]->ploth_polysymbol)(handle,ipts,symbols[sym],ix,iy);
       free(ix); free(iy);
   }} else for (i=0; i<pts; i++) plot_point(handle,X[i],Y[i]);
}

static void scaling2d(float *xwmin, float *xwmax,
                      float *ywmin, float *ywmax){
    *xwmin=plot.xyzdz[0];    *xwmax=plot.xyzdz[1];
    *ywmin=plot.xyzdz[2];    *ywmax=plot.xyzdz[3];
}

static void init_plcfg(){
  int i;
  plc[a_h].zoomv=0;     plc[a_h].ixoffset=0;  plc[a_h].iyoffset=0;
  plc[a_h].vectplot=0;  plc[a_h].zoomed=FALSE;
  plc[a_h].alpha=0.0;   plc[a_h].beta=0.0;   
  plc[a_h].alpha0=0.0;  plc[a_h].beta0=0.0; 
  plc[a_h].f1=500;      plc[a_h].f2=1000;
  plc[a_h].sx=0;        plc[a_h].sy=0;
  plc[a_h].netl= TRUE;  plc[a_h].netr= TRUE; 

  plc[a_h].autoaxis  = TRUE;
  plc[a_h].scaling   = SC_EXACT;
  plc[a_h].errorbars = FALSE;
  plc[a_h].vectplot  = FALSE;  
  plc[a_h].residue   = TRUE; 
  for (i=0; i< MAXBUF; i++) plc[a_h].selected[i] = FALSE;
  plc[a_h].title[0] = 0;
  plc[a_h].xlabel[0] = 0; strcpy(plc[a_h].xformat,DEFAULTFORMATX);
  plc[a_h].ylabel[0] = 0; strcpy(plc[a_h].yformat,DEFAULTFORMATY);
  plc[a_h].xmin = 0; plc[a_h].xmax = 1; plc[a_h].xstart = 0; plc[a_h].xstep = 0;
  plc[a_h].ymin = 0; plc[a_h].ymax = 1; plc[a_h].ystart = 0; plc[a_h].ystep = 0;
  plc[a_h].xtics = 5; plc[a_h].ytics = 5;
}

void init_plot(int handle,char plotdev[],char *backgroundcolor){
   int devtype, wait;

   init_v_inp();
   if (plotdev[0]=='\0'){ strcpy(plotdev,"default"); }
   wait = FALSE;
   if (! plot_init()){
      printf("Error loading Symbolfile: no symbols in Plot");
      wait = TRUE;   }
   devtype = PLOT_DEV_X11;
   if (devtype == PLOT_DEV_NO){
      a_h = -1;
      wait = TRUE;
   }else{
      
  /*    printf("1plot_open  winposx=%i  winposy=%i\n",winposx,winposy);*/
      a_h = plot_open(handle,devtype,winsizex,winsizey,
                winposx,winposy,wdescription,backgroundcolor);
      if (a_h < 0){
         printf(" Error initializing Plot\n");
         wait = TRUE;
      }else{
	 init_plcfg();
	 is_window_device = 
         plot_capabilities(a_h,PLOT_CAPABILITY_WINDOWS);
      }

   }
   taste_warten(wait);
}

static void round_scale(float *u,float *o) {
   float fact, diff, oo, uu;
   oo = *o; uu = *u;
   diff = oo - uu;
   fact = 1;
   while (diff < 10)  { diff = diff*10; fact = fact/10;}
   while (diff > 100) { diff = diff/10; fact = fact*10;}
   *u = floor( uu / fact ) * fact;
   *o = ceil( oo / fact ) * fact;
   while( *o <= oo) *o = *o + fact;
   while( *u >= uu) *u = *u - fact;
}

void do_scaling(Scaling scale){
   float tmp;
   if (scale == SC_AXIS){
      if ((plc[a_h].xstep <= 0.0) || (plc[a_h].ystep <= 0.0)) return;
      if (plc[a_h].xmax < plc[a_h].xstart) return;
      if (plc[a_h].ymax < plc[a_h].ystart) return;
      if (((plc[a_h].xmax - plc[a_h].xstart)/plc[a_h].xstep) > 20) return;
      if (((plc[a_h].ymax - plc[a_h].ystart)/plc[a_h].ystep) > 20) return;
      plc[a_h].autoaxis = FALSE;
      plc[a_h].scaling = SC_OFF;

      plc[a_h].xmin = plc[a_h].xstart; plc[a_h].ymin = plc[a_h].ystart;
      tmp = plc[a_h].xmin;
      do { tmp = tmp + plc[a_h].xstep; } while(tmp < plc[a_h].xmax);
      plc[a_h].xmax = tmp;
      tmp = plc[a_h].ymin;
      do { tmp = tmp + plc[a_h].ystep; } while(tmp < plc[a_h].ymax);
      plc[a_h].ymax = tmp;
      return;
   }

   scaling2d(&plc[a_h].xmin,&plc[a_h].xmax,&plc[a_h].ymin,&plc[a_h].ymax);
   if (scale == SC_ROUND){
      round_scale(&plc[a_h].xmin,&plc[a_h].xmax);
      round_scale(&plc[a_h].ymin,&plc[a_h].ymax);
}}

static void write_number(int handle, int x, int y, float v,
			 int hor, int ver, char format[])
{ char outstr[80];

  plot_settextalign(handle,hor,ver);
  sprintf(outstr,format,v);
  plot_writetext(handle,TRUE,x,y,outstr);
}

static void simple_axis(int handle,int x1, int y1, int x2, int y2){
   int i,xi[TICS],yi[TICS];
  for (i=0;i<TICS;i++){
     xi[i] = i /(TICS-1.0) *(x2-x1) + x1;
     yi[i] = i /(TICS-1.0) *(y2-y1) + y1;
  }
  for (i=0;i<TICS;i++)
     plot_line(handle,xi[i],y1-frame,xi[i],y1-frame+ticlen);
  for (i=0;i<TICS;i++)
     plot_line(handle,xi[i],y2+frame-ticlen,xi[i],y2+frame);
  for (i=0;i<TICS;i++)
     plot_line(handle,x1-frame,yi[i],x1-frame+ticlen,yi[i]);
  for (i=0;i<TICS;i++)
     plot_line(handle,x2+frame-ticlen,yi[i],x2+frame,yi[i]);
   write_number(handle,sizex*LEFT_FRAME-3*frame,
                       sizey*BOTTOM_FRAME-3*frame,plc[a_h].xmin,
                PLOT_LEFTTEXT,PLOT_TOPTEXT,plc[a_h].xformat);
   write_number(handle,sizex*RIGHT_FRAME,
		       sizey*BOTTOM_FRAME-3*frame,plc[a_h].xmax,
                PLOT_RIGHTTEXT,PLOT_TOPTEXT,plc[a_h].xformat);
   write_number(handle,sizex*LEFT_FRAME-iframe*frame,
		       sizey*BOTTOM_FRAME,plc[a_h].ymin,
                PLOT_RIGHTTEXT,PLOT_CENTERTEXT,plc[a_h].yformat);
   write_number(handle,sizex*LEFT_FRAME-iframe*frame,
		       sizey*TOP_FRAME,plc[a_h].ymax,
                PLOT_RIGHTTEXT,PLOT_CENTERTEXT,plc[a_h].yformat);
}

static void draw_tics(int handle, int kk, int x_or_y, int is_long, float val,
		      int pos1, int pos2){
   int len;
   if (is_long) len = 2*ticlen; else len = ticlen;
   if (x_or_y){   /* Y-Axis */
     plot_line(handle,pos1-frame-len,kk,pos1-frame+len,kk);
     plot_line(handle,pos2+frame-len,kk,pos2+frame+len,kk);
     if (is_long) write_number(handle,sizex*LEFT_FRAME-3*frame,kk,val,
                PLOT_RIGHTTEXT,PLOT_CENTERTEXT,plc[a_h].yformat);
   }else{         /* X-Axis */
     plot_line(handle,kk,pos1-frame-len,kk,pos1-frame+len);
     plot_line(handle,kk,pos2+frame-len,kk,pos2+frame+len);
     if (is_long) write_number(handle,kk+FRAME,sizey*BOTTOM_FRAME-3*frame,val,
                PLOT_RIGHTTEXT,PLOT_TOPTEXT,plc[a_h].xformat);
   }
}


static void customized_axis(handle,x1,y1,x2,y2){
   int xk, yk, i, istepx, istepy, istartx, istarty,
       ixnull, iynull;
   float val, ticx, ticy;
   plot_setview(handle,sizex*LEFT_FRAME,sizey*BOTTOM_FRAME,
                sizex*RIGHT_FRAME, sizey*TOP_FRAME);
   plot_setwin(handle,plc[a_h].xmin,plc[a_h].ymin,plc[a_h].xmax,plc[a_h].ymax);
   world_to_screen(handle,0.0,0.0,&ixnull,&iynull);
   world_to_screen(handle,plc[a_h].xstart,plc[a_h].ystart,&istartx,&istarty);
   world_to_screen(handle,plc[a_h].xstep,plc[a_h].ystep,&istepx,&istepy);
   istepx = istepx - ixnull; istepy = istepy - iynull;
   ticx = (float)istepx / plc[a_h].xtics;
   ticy = (float)istepy / plc[a_h].ytics;

   plot_setwin(handle,0,0,sizex,sizey);
   plot_setview(handle,0,0,sizex,sizey);
   val = plc[a_h].xstart;
   for (xk=istartx; xk<=x2+FRAME; xk = xk+istepx){
     draw_tics(handle,xk,0,1,val,y1,y2);
     for (i=1; i<plc[a_h].xtics; i++)
       if (xk+ticx*i < x2)
         draw_tics(handle,(int)(xk+ticx*i),0,0,0.0,y1,y2);
     val = val + plc[a_h].xstep;
   }
   val = plc[a_h].ystart;
   for (yk=istarty; yk<=y2+FRAME; yk = yk+istepy){
     draw_tics(handle,yk,1,1,val,x1,x2);
     for (i=1; i<plc[a_h].ytics; i++)
       if (yk+ticy*i < y2)
         draw_tics(handle,(int)(yk+ticy*i),1,0,0.0,x1,x2);
     val = val + plc[a_h].ystep;
   }
}


static int check_axis(){
   if (plc[a_h].xtics < 1) plc[a_h].xtics = 1;
   if (plc[a_h].ytics < 1) plc[a_h].ytics = 1;

   if ((plc[a_h].xstep <= 0.0) || (plc[a_h].ystep <= 0.0)) return FALSE;

   if (plc[a_h].xmin > plc[a_h].xstart) return FALSE;
   if (plc[a_h].xmax < (plc[a_h].xstart+plc[a_h].xstep)) return FALSE;
   if (((plc[a_h].xmax - plc[a_h].xmin)/plc[a_h].xstep) > 20) return FALSE;

   if (plc[a_h].ymin > plc[a_h].ystart) return FALSE;
   if (plc[a_h].ymax < (plc[a_h].ystart+plc[a_h].ystep)) return FALSE;
   if (((plc[a_h].ymax - plc[a_h].ymin)/plc[a_h].ystep) > 20) return FALSE;

   return TRUE;
}


static void draw_frame(int handle,int x1, int y1, int x2, int y2)
{ int use_the_axis;
  if (! plc[a_h].autoaxis){
     use_the_axis = check_axis();
     if (! use_the_axis)
        printf(" Warning: Customized axis could not be used\n");
  } else use_the_axis = FALSE;
  plot_line(handle,x1-frame,y1-frame,x2+frame,y1-frame);
  plot_line(handle,x1-frame,y2+frame,x2+frame,y2+frame);
  plot_line(handle,x1-frame,y1-frame,x1-frame,y2+frame);
  plot_line(handle,x2+frame,y1-frame,x2+frame,y2+frame);
  if (use_the_axis) customized_axis(handle,x1,y1,x2,y2);
               else simple_axis(handle,x1,y1,x2,y2);
}

static void one_curve_data(int handle, int n_pts,
                      float x[], float y[], float err[]){
   int i;
       plot_polysymbol(handle,n_pts,1,x,y);
      if (plc[a_h].errorbars){ 
       for (i=0; i<n_pts; i++)
       plot_line(handle,x[i],y[i]-err[i],x[i],y[i]+err[i]);
      }
}

static void one_curve_theory(int handle, int n_pts,
                      float x[], float y[]){
   plot_polyline(handle,n_pts,x,y); /* Theory or Subspectrum */
}

static void mos_plot(int handle){
    int k, km,i,col;
   float *xt,*yt;
  xt = malloc(MAXN*sizeof(float));
  yt = malloc(MAXN*sizeof(float));
   if (plot_data){plot_setfgbyname(handle,plot.datacolor);
   one_curve_data(handle,n_data,pxd,pyd,pyderr);}
   if(plc[a_h].residue){plot_setfgbyname(handle,plot.rescolor);
     /* printf(" plot.nchanplot[3]  %i \n",plot.nchanplot[4]);*/
    one_curve_data(handle,n_data,pxd,pyres,pyderr);
    }       
   if (have_theo){
      n_sub=plot.n_subspectra;
/*   printf(" +++++++++++++++++++++++theo n_sub  %i \n",n_sub);*/
      /*   printf(" have_theo %i\n",have_theo);
	   printf(" plot_theo %i\n",plot_theo);*/
      if (plot_theo){plot_setfgbyname(handle,plot.thcolor);
      km=0;
      for(k=0; k<n_theo;k++){
          xt[km]=pxt[k]; 
          yt[km]=pyt[k];
	  km=km+1;
	  /*printf("j=%d jm=%d  x=%f y=%f \n",k,km,pxt[k],pyt[k]);*/
            if(pyt[k] == -1.0){
	    one_curve_theory(handle,km-1,xt,yt);km=0;}
      }
      one_curve_theory(handle,km,pxt,pyt);}
      if (plot_subspectra){

      if (n_sub > 0) {
        for (k=0; k<n_sub; k++){
	  if(pl_subsp[k]){col=plot.ncolor[k];
             plot_setfgbyname(handle,win.tabcolor[col]);
             one_curve_theory(handle,n_theo,pxt,psub[k]);
	  }
   }}}}
   free(xt); free(yt);
}

void draw_frame2d(int handle,int sx, int sy){

float xp,yp,wx,wy,cmpwx,cmpwy,xm,ym;
int ii,jj,ji,npoints,jstring,ipfn,ipfo;

XPoint *npoly;
npoly = malloc(4*MAXN*sizeof(short int));
 

 wx=(float)(sx)/stwinsizex;
 wy=(float)(sy)/stwinsizey;

/* frame and axes description ----------------------*/
	xm=sx/2;;
        ym=sy/2;
        cmpwx=cmpixel*wx*pixwin_cm;
	cmpwy=cmpixel*wy*pixwin_cm;

           ipfo=-1;
	   ji=0;
           while(fabs(plot.ltext[ji])>0){
	       xp=(plot.op[ji].xp*cmpwx-xoffset*wx)*zoomf;
	       yp=(plot.op[ji].yp*cmpwy+yoffset*wy)*zoomf;
            jstring=plot.ltext[ji];
            if(ji == 0) ii=0;  else  ii=plot.ltext[ji-1];
	    jj=ii;
            while(jj<jstring){
             ii=0;
	     npoly[ii].x=sx-(xm-(xm-plot.vtxt[jj].y*cmpwx)*zoomf+xp);
	     npoly[ii].y=sy-(ym-(ym-plot.vtxt[jj].x*cmpwy)*zoomf+yp);
            ipfn=plot.ifeder[jj+1];
            if(ipfn != ipfo){
		/*printf("jj+1=%d ipfn=%d ipfo=%d color=%s\n",jj+1,ipfn,ipfo,
                  win.tabcolor[ipfn]);*/
             if(ipfn > -1) plot_setfgbyname(handle,win.tabcolor[ipfn]);
            }
            ipfo=ipfn;
            npoints=0;
	    while(plot.ifeder[jj+1]>-1){
	     ii=ii+1;
	     npoly[ii].x=sx-(xm-(xm-plot.vtxt[jj+1].y*cmpwx)*zoomf+xp);
	     npoly[ii].y=sy-(ym-(ym-plot.vtxt[jj+1].x*cmpwy)*zoomf+yp);
             npoints=ii+1; jj=jj+1; 
             /*printf("npoints=%d\n",npoints);*/
            };
            jj=jj+1;
            plot_polylinevc(handle,npoints,npoly);
           }
           ji=ji+1;
          };
      free(npoly);
/* end of axes description ----------------------*/
}
void draw_exp2d(int handle,int sx, int sy){
float xp,yp,wx,wy,cmpwx,cmpwy,xm,ym,yval,b0x,b1x,b0y,b1y;
int j;
short int nx,ny,nxe,nye;     

        plot_setfgbyname(handle,plot.datacolor);
	zoomf=pow(1.05,plc[a_h].zoomv);
        xoffset=plc[a_h].ixoffset;
        yoffset=plc[a_h].iyoffset;

        wx=(float)(sx)/stwinsizex;
        wy=(float)(sy)/stwinsizey;

	xm=sx/2;;
        ym=sy/2;
        cmpwx=cmpixel*wx;
	cmpwy=cmpixel*wy;
        b1x=1./plot.xyzdz[5]*pixwin_cm;
        b0x=-plot.xyzdz[4]*b1x;
        b1y=1./plot.xyzdz[7]*pixwin_cm;
        b0y=-plot.xyzdz[6]*b1y;
	xp=(-xoffset*wx)*zoomf;
	yp=(yoffset*wy)*zoomf;
      if(plot_data){
        for(j=0; j<n_data; j++){ 
         nx=sx-(xm-(xm-(b0x+b1x*pxd[j])*cmpwx)*zoomf+xp);
         nxe=nx;
        if(plc[a_h].errorbars){
 	  yval=b0y+b1y*(pyd[j]-pyderr[j]);
         ny=sy-(ym-(ym-yval*cmpwy)*zoomf+yp);
 	  yval=b0y+b1y*(pyd[j]+pyderr[j]);
         nye=sy-(ym-(ym-yval*cmpwy)*zoomf+yp);
         plot_linevc(handle,nx,ny,nxe,nye);
        }else{
         ny=sy-(ym-(ym-(b0y+b1y*pyd[j])*cmpwy)*zoomf+yp);
 	  plot_symbolvc(handle,0,nx,ny);}
        }} 
      if(plc[a_h].residue){
        plot_setfgbyname(handle,plot.rescolor);
        for(j=0; j<n_data; j++){ 
         nx=sx-(xm-(xm-(b0x+b1x*pxd[j])*cmpwx)*zoomf+xp);
         nxe=nx;
 	 yval=b0y+b1y*(pyres[j]-pyderr[j]);
         ny=sy-(ym-(ym-yval*cmpwy)*zoomf+yp);
 	  yval=b0y+b1y*(pyres[j]+pyderr[j]);
         nye=sy-(ym-(ym-yval*cmpwy)*zoomf+yp);
         plot_linevc(handle,nx,ny,nxe,nye);}
      }  
}

void draw_theo2d(int handle,int sx, int sy){
float xp,yp,wx,wy,cmpwx,cmpwy,xm,ym,b0x,b1x,b0y,b1y;
int k,j,jm,col;
XPoint *npoly;
npoly = malloc(2*MAXN*sizeof(short int));

        plot_setfgbyname(handle,plot.thcolor);
	zoomf=pow(1.05,plc[a_h].zoomv);
        xoffset=plc[a_h].ixoffset;
        yoffset=plc[a_h].iyoffset;

        wx=(float)(sx)/stwinsizex;
        wy=(float)(sy)/stwinsizey;
	xm=sx/2;;
        ym=sy/2;
        cmpwx=cmpixel*wx;
	cmpwy=cmpixel*wy;
        b1x=1./plot.xyzdz[5]*pixwin_cm;
        b0x=-plot.xyzdz[4]*b1x;
        b1y=1./plot.xyzdz[7]*pixwin_cm;
        b0y=-plot.xyzdz[6]*b1y;
	xp=(-xoffset*wx)*zoomf;
	yp=(yoffset*wy)*zoomf;

       if (plot_theo){
	   jm=0;
        for(j=0; j<n_theo; j++){ 
         npoly[jm].y=sy-(ym-(ym-(b0y+b1y*pyt[j])*cmpwy)*zoomf+yp);
         npoly[jm].x=sx-(xm-(xm-(b0x+b1x*pxt[j])*cmpwx)*zoomf+xp);
         /*printf("j=%d jm=%d  x=%f y=%f \n",j,jm,pxt[j],pyt[j]);*/
         jm=jm+1;
	 if(pyt[j] == -1) {
         plot_polylinevc(handle,jm-1,npoly);jm=0;}
        } 
 	 plot_polylinevc(handle,jm,npoly);
       }

       if (plot_subspectra){

        if (n_sub > 0) {
         for (k=0; k<n_sub; k++){
	   if(pl_subsp[k]){col=plot.ncolor[k];
	   /*printf("col=%d  color=%s\n",col,win.tabcolor[col]);*/
             plot_setfgbyname(handle,win.tabcolor[col]);
             for(j=0; j<n_theo; j++){
             npoly[j].x=sx-(xm-(xm-(b0x+b1x*pxt[j])*cmpwx)*zoomf+xp);
             npoly[j].y=sy-(ym-(ym-(b0y+b1y*psub[k][j])*cmpwy)*zoomf+yp);}
             plot_polylinevc(handle,n_theo,npoly);
	   }}}
       }
      free(npoly);
}

static void do_the_plot2d(int handle){

   plot_getdev(handle,&winposx,&winposy,&sizex,&sizey);
   winposx=winposx-2;
   winposy=winposy-24;
/* plot_getdev returns wrong values*/

   size_factor = sizex / 1000;
   if (size_factor < 1.0) size_factor = 1.0;

   plot_setsymsize(handle,size_factor);
   frame = FRAME * size_factor;
   ticlen = TICLEN * size_factor;
   plot_clear(handle);
   plot_setmode(handle,PLOT_MODE_OVER);
   plot_setwin(handle,0,0,sizex,sizey);
   plot_setview(handle,0,0,sizex,sizey);

   if(size!=sizey) plotd_sel_font(handle,sizey);
   size=sizey;
   plot_settextalign(handle, PLOT_LEFTTEXT,PLOT_BOTTOMTEXT); 
   
   plot_setview(handle,sizex*LEFT_FRAME,sizey*BOTTOM_FRAME,
                sizex*RIGHT_FRAME, sizey*TOP_FRAME);
   plot_setclip(handle,TRUE);
   if(have_data) draw_exp2d(handle, sizex, sizey);
   if(have_theo) draw_theo2d(handle, sizex, sizey);
   draw_frame2d(handle, sizex, sizey);
}

static void do_the_plot(int handle){
   int i,j,idy;
   char ch;
   plot_getdev(handle,&winposx,&winposy,&sizex,&sizey);

/* plot_getdev returns wrong values*/
   winposx=winposx-2;
   winposy=winposy-24;
/* plot_getdev returns wrong values*/

   size_factor = sizex / 1000;
   if (size_factor < 1.0) size_factor = 1.0;
   /*printf("handle=%d sizex=%d sizey=%d\n",handle,sizex,sizey);*/

   plot_setsymsize(handle,size_factor);
   frame = FRAME * size_factor;
   ticlen = TICLEN * size_factor;
   plot_clear(handle); 
   plot_setmode(handle,PLOT_MODE_OVER);
   plot_setwin(handle,0,0,sizex,sizey);
   plot_setview(handle,0,0,sizex,sizey);

   if(size!=sizey) plotd_sel_font(handle,sizey);
   size=sizey; 

    plot_setfgbyname(handle,plot.fgcolor);
    draw_frame(handle,sizex*LEFT_FRAME,sizey*BOTTOM_FRAME,
	      sizex*RIGHT_FRAME, sizey*TOP_FRAME);
 
   plot_settextalign(handle, PLOT_LEFTTEXT,PLOT_BOTTOMTEXT); 
   plot_writetext(handle, FALSE,
	      sizex*LEFT_FRAME,
	      sizey*TITLE_POSY,plc[a_h].title);
   plot_settextalign(handle, PLOT_RIGHTTEXT,PLOT_BOTTOMTEXT);
   plot_writetext(handle, FALSE,
	      sizex*RIGHT_FRAME,
	      sizey*XLABEL_POSY,plc[a_h].xlabel);
   plot_settextalign(handle, PLOT_LEFTTEXT,PLOT_BOTTOMTEXT); 

   i=sizey/100.;if(i<=2) i=2; if(i>=6) i=6;
   switch(i){ 
     case 2: idy=6;break;
     case 3: idy=7;break;
     case 4: idy=10;break;
     case 5: idy=16;break;
     case 6: idy=20;break;
   };
   /*ch=plc[a_h].ylabel[1];printf("len ch=%d \n",strlen(&ch));*/

   i=strlen(plc[a_h].ylabel);
   j=0;
       while(j<i){ch=plc[a_h].ylabel[j];
         plot_writechar(handle, FALSE,4*FRAME,
		  sizey*YLABEL_POSY-idy*j,&ch);j++;}
   plot_setview(handle,sizex*LEFT_FRAME,sizey*BOTTOM_FRAME,
                sizex*RIGHT_FRAME, sizey*TOP_FRAME);
    plot_setwin(handle,plc[a_h].xmin,plc[a_h].ymin,plc[a_h].xmax,plc[a_h].ymax);
    /*printf("xi=%10.2f  yi=%10.2f  xa=%10.2f  ya=%10.2f\n  ",
      plc[a_h].xmin,plc[a_h].ymin,plc[a_h].xmax,plc[a_h].ymax);*/
   plot_setclip(handle,TRUE);
   mos_plot(handle);
}

void draw_frame3d(int handle){
    float alpha,beta; 
    int f1,f2,sx,sy;

#define bottom 0
float gx,gy,gz,hf2,xp,yp,zp,ox,oy,oz,vx,vy,vz,
      sina,cosa,sinb,cosb,sinc,cosc,cosx,sinx,cosy,siny,cosz,sinz,
    vertical,xi,cmpwx,cmpwy,
      rx,ry,f1rxry,f1f2rxry,wx,wy,height,hheight,hwidth,
      axis[3],ax[3],rot[3][3];
int i,j,ii,jj,ji,npoints,ip,jstring,ipfn,ipfo;
     
XPoint *npoly;
npoly = malloc(4*MAXN*sizeof(short int));
 
       alpha=plc[a_h].alpha;
       beta =plc[a_h].beta ;
       sx   =plc[a_h].sx   ;
       sy   =plc[a_h].sy   ;
       f1   =plc[a_h].f1   ; 
       f2   =plc[a_h].f2   ; 

/* rotation matrix rot*/
		cosa=cos(alpha); sina=sin(alpha);
		cosb=cos(beta); sinb=sin(beta);

		/*printf("cosb=%10.4f sinb=%10.4f ",cosb,sinb);
		  printf("cosa=%10.4f sina=%10.4f \n",cosa,sina);*/

                rot[0][0]=sinb*sinb*(1.-cosa)+cosa;
                rot[2][2]=cosb*cosb*(1.-cosa)+cosa;
                rot[0][1]=cosb*sina; rot[1][0]=-rot[0][1];
                rot[0][2]=sinb*cosb*(1.-cosa);rot[2][0]=rot[0][2];
                rot[1][1]=cosa;
                rot[1][2]=sinb*sina; rot[2][1]=-rot[1][2];


/*coordinates xc,yc,zc  error ez   trans formed to ax[3]
winstwd=; */

               /* z-coordinate of observer*/
		hf2=100;
                 /* screen --- observer*/
                vertical=(float)(sizey-bottom);
                rx=(float)sizex/stwinsizex;             
                ry=(float)sizey/stwinsizey;
                f1rxry=(float)f1*rx*ry;
                f1f2rxry=(float)(f1+f2)*rx*ry;

		/*dx=plc[a_h].xmax-plc[a_h].xmin;
		  dy=plc[a_h].ymax-plc[a_h].ymin;  10.01.08     
                dx=1.; dy=1.0;
  axes frame --------------------------------------------------
            if(dx > dy){
             wx=0.9*stwinsizex/2;
             wy=wx*dy/dx;
		 }else{*/
	     wx=0.9*stwinsizex/2;
             wy=wx;
             /*wx=wx*dx/dy;*/
		  

             height=stwinsizey;             
             hwidth=stwinsizex/2;
             hheight=height-bottom;
	    npoints=6;
            pdraw[0].x=-wx; pdraw[0].y=-wy; pdraw[0].z=0.;
            pdraw[1].x=+wx; pdraw[1].y=-wy; pdraw[1].z=0.;
            pdraw[2].x=+wx; pdraw[2].y=+wy; pdraw[2].z=0.;
            pdraw[3].x=-wx; pdraw[3].y=+wy; pdraw[3].z=0.;
            pdraw[4].x=-wx; pdraw[4].y=-wy; pdraw[4].z=0.;
            pdraw[5].x=-wx; pdraw[5].y=-wy; pdraw[5].z=height;

            for(ip=0;ip<npoints;ip++){
             axis[0]= cosb*pdraw[ip].x+sinb*pdraw[ip].z;
             axis[1]= pdraw[ip].y;
             axis[2]=-sinb*pdraw[ip].x+cosb*pdraw[ip].z;                   
             for(i=0;i<3;i++){ax[i]=0.;
              for(j=0;j<3;j++)ax[i]=ax[i]+rot[j][i]*axis[j];}
             xi=(f1f2rxry-ax[0])/(f1rxry-ax[0]);
             npoly[ip].x=sx+(-ax[1]*(1.-xi)/xi+hwidth)*rx;
             npoly[ip].y=sy+(hheight*(cosb+1.)/2.-(hf2-ax[2]*(1.-xi))/xi)*ry;
             /*printf("ip=%d  x=%d  y=%d \n",ip,npoly[ip].x,npoly[ip].y);*/
            }
            
            plot_polylinevc(handle,npoints,npoly);


	    npoints=2;
            pdraw[0].x=-wx; pdraw[0].y=0.; pdraw[0].z=0.;
            pdraw[1].x=-wx+10; pdraw[1].y=0.; pdraw[1].z=0.;

            for(ip=0;ip<npoints;ip++){
              axis[0]= cosb*pdraw[ip].x+sinb*pdraw[ip].z;
              axis[1]= pdraw[ip].y;
              axis[2]=-sinb*pdraw[ip].x+cosb*pdraw[ip].z;                   
              for(i=0;i<3;i++){ax[i]=0.;
                 for(j=0;j<3;j++)ax[i]=ax[i]+rot[j][i]*axis[j];}
              xi=(f1f2rxry-ax[0])/(f1rxry-ax[0]);
	      npoly[ip].x=sx+(-ax[1]*(1.-xi)/xi+hwidth)*rx;
              npoly[ip].y=sy+(hheight*(cosb+1.)/2.-(hf2-ax[2]*(1.-xi))/xi)*ry;
            }
            plot_polylinevc(handle,npoints,npoly);/* tick1 */

	    npoints=2;
            pdraw[0].x=wx; pdraw[0].y=0.; pdraw[0].z=0.;
            pdraw[1].x=wx-10; pdraw[1].y=0.; pdraw[1].z=0.;

            for(ip=0;ip<npoints;ip++){
              axis[0]= cosb*pdraw[ip].x+sinb*pdraw[ip].z;
              axis[1]= pdraw[ip].y;
              axis[2]=-sinb*pdraw[ip].x+cosb*pdraw[ip].z;                   
              for(i=0;i<3;i++){ax[i]=0.;
                 for(j=0;j<3;j++)ax[i]=ax[i]+rot[j][i]*axis[j];}
              xi=(f1f2rxry-ax[0])/(f1rxry-ax[0]);
	      npoly[ip].x=sx+(-ax[1]*(1.-xi)/xi+hwidth)*rx;
              npoly[ip].y=sy+(hheight*(cosb+1.)/2.-(hf2-ax[2]*(1.-xi))/xi)*ry;
            }
            plot_polylinevc(handle,npoints,npoly);/* tick2 */
 

	    npoints=2;
            pdraw[0].x=0.; pdraw[0].y=-wy; pdraw[0].z=0.;
            pdraw[1].x=0.; pdraw[1].y=-wy+10; pdraw[1].z=0.;

            for(ip=0;ip<npoints;ip++){
              axis[0]= cosb*pdraw[ip].x+sinb*pdraw[ip].z;
              axis[1]= pdraw[ip].y;
              axis[2]=-sinb*pdraw[ip].x+cosb*pdraw[ip].z;                   
              for(i=0;i<3;i++){ax[i]=0.;
                 for(j=0;j<3;j++)ax[i]=ax[i]+rot[j][i]*axis[j];}
              xi=(f1f2rxry-ax[0])/(f1rxry-ax[0]);
	      npoly[ip].x=sx+(-ax[1]*(1.-xi)/xi+hwidth)*rx;
              npoly[ip].y=sy+(hheight*(cosb+1.)/2.-(hf2-ax[2]*(1.-xi))/xi)*ry;
            }
            plot_polylinevc(handle,npoints,npoly);/* tick3 */

	    npoints=2;
            pdraw[0].x=0.; pdraw[0].y=wy; pdraw[0].z=0.;
            pdraw[1].x=0.; pdraw[1].y=wy-10; pdraw[1].z=0.;

            for(ip=0;ip<npoints;ip++){
              axis[0]= cosb*pdraw[ip].x+sinb*pdraw[ip].z;
              axis[1]= pdraw[ip].y;
              axis[2]=-sinb*pdraw[ip].x+cosb*pdraw[ip].z;                   
              for(i=0;i<3;i++){ax[i]=0.;
                 for(j=0;j<3;j++)ax[i]=ax[i]+rot[j][i]*axis[j];}
              xi=(f1f2rxry-ax[0])/(f1rxry-ax[0]);
	      npoly[ip].x=sx+(-ax[1]*(1.-xi)/xi+hwidth)*rx;
              npoly[ip].y=sy+(hheight*(cosb+1.)/2.-(hf2-ax[2]*(1.-xi))/xi)*ry;
            }
            plot_polylinevc(handle,npoints,npoly);/* tick4 */

	    npoints=3;
            pdraw[0].x=-wx; pdraw[0].y=-wy+10; pdraw[0].z=hheight*0.75;
            pdraw[1].x=-wx; pdraw[1].y=-wy; pdraw[1].z=hheight*0.75;
            pdraw[2].x=-wx+10; pdraw[2].y=-wy; pdraw[2].z=hheight*0.75;

            for(ip=0;ip<npoints;ip++){
              axis[0]= cosb*pdraw[ip].x+sinb*pdraw[ip].z;
              axis[1]= pdraw[ip].y;
              axis[2]=-sinb*pdraw[ip].x+cosb*pdraw[ip].z;                   
              for(i=0;i<3;i++){ax[i]=0.;
                 for(j=0;j<3;j++)ax[i]=ax[i]+rot[j][i]*axis[j];}
              xi=(f1f2rxry-ax[0])/(f1rxry-ax[0]);
	      npoly[ip].x=sx+(-ax[1]*(1.-xi)/xi+hwidth)*rx;
              npoly[ip].y=sy+(hheight*(cosb+1.)/2.-(hf2-ax[2]*(1.-xi))/xi)*ry;
            }
            plot_polylinevc(handle,npoints,npoly);/* tick z-axis */
        
/* axes description ----------------------*/
/*	  the x,y-plane of vectortext is taken as the y,z-plane
          rotation phiy about the y-axis*/
/*        rotation alphaz about the z-axis*/
/*        ux= ax*cos(alphaz)+ay*sin(alphaz)
          uy=-ax*sin(alphaz)+ay*cos(alphaz)
          uz= az
          rotation phix about the x-axis
          vx= ux
          vy= uy*cos(phix)+uz*sin(phix)
          vz=-uy*sin(phix)+uz*cos(phix)*/

/*---------^----begin of title ---------------------*/
           cmpwx=cmpixel*rx*pixwin_cm;
           cmpwy=cmpixel*ry*pixwin_cm;
           cmpwx=rx*pixwin_cm;
           cmpwy=ry*pixwin_cm;
           ipfo=-1;
	   ji=0;
           xp=plot.op[ji].xp;
	   yp=sizey-plot.op[ji].yp;
            jstring=-plot.ltext[ji];
            if(jstring<0)jstring=-jstring;
	    jj=0;
            while(jj<jstring){
             ii=0;
	     npoly[ii].x=plot.vtxt[jj].y*cmpwx+xp;
	     npoly[ii].y=plot.vtxt[jj].x*cmpwy+yp;
            ipfn=plot.ifeder[jj+1];
            if(ipfn != ipfo){
	     /*printf("jj+1=%d ipfn=%d ipfo=%d color=%s\n",jj+1,ipfn,ipfo,
                  win.tabcolor[ipfn]);*/
             if(ipfn > -1) plot_setfgbyname(handle,win.tabcolor[ipfn]);
            }
            ipfo=ipfn;
            npoints=0;
	    while(plot.ifeder[jj+1]>-1){
	     ii=ii+1;
	     npoly[ii].x=plot.vtxt[jj+1].y*cmpwx+xp;
	     npoly[ii].y=plot.vtxt[jj+1].x*cmpwy+yp;
             npoints=ii+1; jj=jj+1; 
	     /*printf("x=%d y=%d\n",npoly[ii].x,npoly[ii].x);*/
            };
            jj=jj+1;
            if(!notitle)plot_polylinevc(handle,npoints,npoly);
           }
           
           ji=ji+1;

/*-------------end of title ---------------------*/

           ipfo=-1;
	   /*ji=0;*/
           while(fabs(plot.ltext[ji])>0){
	    xp=(plot.op[ji].xp-0.5)*wx*2.+plot.op[ji].strsx*cmpixel;
	    yp=(plot.op[ji].yp-0.5)*wy*2.+plot.op[ji].strsy*cmpixel;
	    zp=plot.op[ji].zp*hheight;
            cosx=cos(plot.op[ji].phix); sinx=sin(plot.op[ji].phix);
            cosy=cos(plot.op[ji].phiy); siny=sin(plot.op[ji].phiy);
            cosz=cos(plot.op[ji].phiz); sinz=sin(plot.op[ji].phiz);
            jstring=plot.ltext[ji];
	    /*printf("jj=%d x=%8.1f y=%8.1f \n",ji,....);*/
            if(ji == 0) ii=0;  else  ii=plot.ltext[ji-1];
            if(ii<0){ii=-ii;};
            if(jstring<0){jstring=-jstring;
	    cosc=cosa; sinc=sina;}else{cosc=1.;sinc=0.;}

	    jj=ii;
            while(jj<jstring){
             ii=0;
 	     ox=plot.vtxt[jj].y*siny;
             oy=plot.vtxt[jj].x;
             oz=plot.vtxt[jj].y*cosy;            
             vx= ox*cosz+oy*sinz;
             vy= (-ox*sinz+oy*cosz)*cosx+oz*sinx;
             vz=-(-ox*sinz+oy*cosz)*sinx+oz*cosx;

	     gx=( cosc*vx+sinc*vy)*cmpixel+xp;
	     gy=(-sinc*vx+cosc*vy)*cmpixel+yp;
	     gz=vz*cmpixel+zp;
                 axis[0]= cosb*gx+sinb*gz;
                 axis[1]= gy;
                 axis[2]=-sinb*gx+cosb*gz;                   
                 for(i=0;i<3;i++){
                 ax[i]=0.; for(j=0;j<3;j++)ax[i]=ax[i]+rot[j][i]*axis[j];}
 	   	 xi=(f1f2rxry-ax[0])/(f1rxry-ax[0]);
	    npoly[ii].x=sx+(-ax[1]*(1.-xi)/xi+hwidth)*rx;
            npoly[ii].y=sy+(hheight*(cosb+1.)/2.-(hf2-ax[2]*(1.-xi))/xi)*ry;

            ipfn=plot.ifeder[jj+1];
            if(ipfn != ipfo){
		/*printf("jj+1=%d ipfn=%d ipfo=%d\n",jj+1,ipfn,ipfo);*/
             if(ipfn > -1) plot_setfgbyname(handle,win.tabcolor[ipfn]);
            }
            ipfo=ipfn;
            npoints=0;
	    while(plot.ifeder[jj+1]>-1){
	     ii=ii+1;
             ox=plot.vtxt[jj+1].y*siny;
             oy=plot.vtxt[jj+1].x;
             oz=plot.vtxt[jj+1].y*cosy;            
             vx= ox*cosz+oy*sinz;
             vy= (-ox*sinz+oy*cosz)*cosx+oz*sinx;
             vz=-(-ox*sinz+oy*cosz)*sinx+oz*cosx;

             gx=( cosc*vx+sinc*vy)*cmpixel+xp;
	     gy=(-sinc*vx+cosc*vy)*cmpixel+yp;
	     gz=vz*cmpixel+zp;
                 axis[0]= cosb*gx+sinb*gz;
                 axis[1]= gy;
                 axis[2]=-sinb*gx+cosb*gz;                   
                 for(i=0;i<3;i++){
                 ax[i]=0.; for(j=0;j<3;j++)ax[i]=ax[i]+rot[j][i]*axis[j];}
 	   	 xi=(f1f2rxry-ax[0])/(f1rxry-ax[0]);
	     npoly[ii].x=sx+(-ax[1]*(1.-xi)/xi+hwidth)*rx;
             npoly[ii].y=sy+(hheight*(cosb+1.)/2.-(hf2-ax[2]*(1.-xi))/xi)*ry;
             npoints=ii+1; jj=jj+1; 
             /*printf("npoints=%d\n",npoints);*/
            };
            jj=jj+1;
            plot_polylinevc(handle,npoints,npoly);
           }
           ji=ji+1;
          };

	    npoints=4;
            pdraw[0].x=-wx+5; pdraw[0].y=-wy+5; pdraw[0].z=0.;
            pdraw[1].x=+wx-5; pdraw[1].y=-wy+5; pdraw[1].z=0.;
            pdraw[2].x=+wx-5; pdraw[2].y=+wy-5; pdraw[2].z=0.;
            pdraw[3].x=-wx+5; pdraw[3].y=+wy-5; pdraw[3].z=0.;

            for(ip=0;ip<npoints;ip++){
              axis[0]= cosb*pdraw[ip].x+sinb*pdraw[ip].z;
              axis[1]= pdraw[ip].y;
              axis[2]=-sinb*pdraw[ip].x+cosb*pdraw[ip].z;                   
              for(i=0;i<3;i++){ax[i]=0.;
                 for(j=0;j<3;j++)ax[i]=ax[i]+rot[j][i]*axis[j];}
              xi=(f1f2rxry-ax[0])/(f1rxry-ax[0]);
	      npoly[ip].x=sx+(-ax[1]*(1.-xi)/xi+hwidth)*rx;
              npoly[ip].y=sy+(hheight*(cosb+1.)/2.-(hf2-ax[2]*(1.-xi))/xi)*ry; 
            }    
            plot_setfgbyname(handle,plot.auxcolor2);
            plot_fillpolygon(handle,npoints,npoly);
            plot_setfgbyname(handle,plot.fgcolor);
/* end of axes description ----------------------*/
      free(npoly);
}

void draw_theo3d(int handle){
    float alpha,beta; 
    int f1,f2,sx,sy;

#define bottom 0
float hf2,sina,cosa,sinb,cosb,vertical,xi,cosbwx,sinbwx,
      xbg,ybg,zbg,zz,cosbwxn,sinbwxn,wyn,cosbhtn,sinbhtn,
      rx,ry,f1rxry,f1f2rxry,wx,wy,height,hheight,hwidth,
      axis[3],ax[3],rot[3][3];
int i,j,jj,npoints,npoint,jrow,jrows,jspec;
 short int lpoints,net;     
XPoint *npoly;
npoly = malloc(4*MAXN*sizeof(short int));
 
       alpha=plc[a_h].alpha;
       beta =plc[a_h].beta ;
       sx   =plc[a_h].sx   ;
       sy   =plc[a_h].sy   ;
       f1   =plc[a_h].f1   ; 
       f2   =plc[a_h].f2   ; 

/* rotation matrix rot*/
		cosa=cos(alpha); sina=sin(alpha);
		cosb=cos(beta); sinb=sin(beta);

		/*printf("cosb=%10.4f sinb=%10.4f ",cosb,sinb);
		  printf("cosa=%10.4f sina=%10.4f \n",cosa,sina);*/

                rot[0][0]=sinb*sinb*(1.-cosa)+cosa;
                rot[2][2]=cosb*cosb*(1.-cosa)+cosa;
                rot[0][1]=cosb*sina; rot[1][0]=-rot[0][1];
                rot[0][2]=sinb*cosb*(1.-cosa);rot[2][0]=rot[0][2];
                rot[1][1]=cosa;
                rot[1][2]=sinb*sina; rot[2][1]=-rot[1][2];
/*coordinates xc,yc,zc  error ez   trans formed to ax[3] winstwd=; */
               /* z-coordinate of observer*/
		hf2=100;
                 /* screen --- observer*/
                vertical=(float)(sizey-bottom);
                rx=(float)sizex/stwinsizex;             
                ry=(float)sizey/stwinsizey;
                f1rxry=(float)f1*rx*ry;
                f1f2rxry=(float)(f1+f2)*rx*ry;

		/*dx=plc[a_h].xmax-plc[a_h].xmin;
		  dy=plc[a_h].ymax-plc[a_h].ymin;  10.01.08     
                dx=1.; dy=1.0;
  axes frame --------------------------------------------------
            if(dx > dy){
             wx=0.9*stwinsizex/2;
             wy=wx*dy/dx;
		 }else{*/
	     wx=0.9*stwinsizex/2;
             wy=wx;
             /*wx=wx*dx/dy;}*/
             height=stwinsizey;             
             hwidth=stwinsizex/2;
             hheight=height-bottom;
             cosbwx=cosb*wx;
             sinbwx=sinb*wx;

             xbg=plot.xyzdz[0];
             ybg=plot.xyzdz[2];
             zbg=plot.xyzdz[4];
             cosbwxn=cosb*wx/plot.xyzdz[1];
             sinbwxn=sinb*wx/plot.xyzdz[1];
             wyn=wy/plot.xyzdz[3];
             cosbhtn=cosb*0.75*height/(plot.xyzdz[5]-plot.xyzdz[4]);
             sinbhtn=sinb*0.75*height/(plot.xyzdz[5]-plot.xyzdz[4]);
           
	    jj=2;          
            npoints=-1;
            zz=p6th[jj]-zbg;
            zz=0.;
            while(p6th[jj]!=-1.){
              npoints=npoints+1;
              axis[0]= cosbwxn*(p6th[jj-2]-xbg)-cosbwx+sinbhtn*zz;
              axis[1]= (p6th[jj-1]-ybg)*wyn-wy;
              axis[2]=-sinbwxn*(p6th[jj-2]-xbg)+sinbwx+cosbhtn*zz;                   
              for(i=0;i<3;i++){ax[i]=0.;
                 for(j=0;j<3;j++)ax[i]=ax[i]+rot[j][i]*axis[j];}
              xi=(f1f2rxry-ax[0])/(f1rxry-ax[0]);
	      npoly[npoints].x=sx+(-ax[1]*(1.-xi)/xi+hwidth)*rx;
              npoly[npoints].y=sy+(hheight*(cosb+1.)/2.-(hf2-ax[2]*(1.-xi))/xi)*ry;     
             jj=jj+3;
            }
            plot_setfgbyname(handle,plot.auxcolor1);
            plot_polylinevc(handle,npoints+1,npoly);
            jj=jj+3;
	    jspec=jj;

            plot_setfgbyname(handle,plot.fgcolor);
	    lpoints=0;net=1;
	   jrow=0;
      	   while(p6th[jj-2]+p6th[jj-1]+p6th[jj]!=-3.){
	       /* printf("=%10.2f =%10.2f =%10.2f\n ",p6th[jj-2],p6th[jj-1],p6th[jj]);*/
	       npoints=-1;jrow=jrow+1;
            while(p6th[jj]!=-1.){
              npoints=npoints+1;
              axis[0]= cosbwxn*(p6th[jj-2]-xbg)-cosbwx+sinbhtn*(p6th[jj]-zbg);
              axis[1]= (p6th[jj-1]-ybg)*wyn-wy;
              axis[2]=-sinbwxn*(p6th[jj-2]-xbg)+sinbwx+cosbhtn*(p6th[jj]-zbg);                   
              for(i=0;i<3;i++){ax[i]=0.;
                 for(j=0;j<3;j++)ax[i]=ax[i]+rot[j][i]*axis[j];}
              xi=(f1f2rxry-ax[0])/(f1rxry-ax[0]);
	      npoly[npoints].x=sx+(-ax[1]*(1.-xi)/xi+hwidth)*rx;
              npoly[npoints].y=sy+(hheight*(cosb+1.)/2.-(hf2-ax[2]*(1.-xi))/xi)*ry;     
             jj=jj+3;
            }

	    if(plc[a_h].netr){
            if(plot.npnt3d[0]==0){for(i=0;i<npoints+1;i++)
		plot_symbolvc(handle,12,npoly[i].x,npoly[i].y);
             }else{ 
		 plot_polylinevc(handle,npoints+1,npoly);}
            }
            jj=jj+3;
            lpoints=lpoints+npoints;
	    if( npoints != lpoints/jrow ) net=0;
           }
           /*printf("npoints=%d \n ",npoints);*/
	   jrows=jrow;

           if(net == 1){
	   npoint=-1;
	   while(npoint<npoints){
	       npoint=npoint+1;
	    jrow=-1;
            while(jrow<jrows){
              jrow=jrow+1;
              jj=jrow*(npoints+2)*3+3*(npoint)+jspec;
              axis[0]= cosbwxn*(p6th[jj-2]-xbg)-cosbwx+sinbhtn*(p6th[jj]-zbg);
              axis[1]= (p6th[jj-1]-ybg)*wyn-wy;
              axis[2]=-sinbwxn*(p6th[jj-2]-xbg)+sinbwx+cosbhtn*(p6th[jj]-zbg);
              for(i=0;i<3;i++){ax[i]=0.;
                 for(j=0;j<3;j++)ax[i]=ax[i]+rot[j][i]*axis[j];}
              xi=(f1f2rxry-ax[0])/(f1rxry-ax[0]);
	      npoly[jrow].x=sx+(-ax[1]*(1.-xi)/xi+hwidth)*rx;
              npoly[jrow].y=sy+(hheight*(cosb+1.)/2.-(hf2-ax[2]*(1.-xi))/xi)*ry;
            }
           if(plc[a_h].netl){
            if(plot.npnt3d[0]==0){for(i=0;i<jrow;i++)
		plot_symbolvc(handle,32,npoly[i].x,npoly[i].y);
             }else{ 
	     plot_polylinevc(handle,jrow,npoly);}
            }
	   }}
      free(npoly);
}
void draw_exp3d(int handle){
    float alpha,beta; 
    int f1,f2,sx,sy;

#define bottom 0
float hf2,sina,cosa,sinb,cosb,vertical,xi,cosbwx,sinbwx,zval,
      xbg,ybg,zbg,cosbwxn,sinbwxn,wyn,cosbhtn,sinbhtn,
      rx,ry,f1rxry,f1f2rxry,wx,wy,height,hheight,hwidth,
      axis[3],ax[3],rot[3][3];
int i,j,jj,npoints;
     
XPoint nval,nvalm;
 
       alpha=plc[a_h].alpha;
       beta =plc[a_h].beta ;
       sx   =plc[a_h].sx   ;
       sy   =plc[a_h].sy   ;
       f1   =plc[a_h].f1   ; 
       f2   =plc[a_h].f2   ; 

/* rotation matrix rot*/
		cosa=cos(alpha); sina=sin(alpha);
		cosb=cos(beta); sinb=sin(beta);

		/*printf("cosb=%10.4f sinb=%10.4f ",cosb,sinb);
		  printf("cosa=%10.4f sina=%10.4f \n",cosa,sina);*/

                rot[0][0]=sinb*sinb*(1.-cosa)+cosa;
                rot[2][2]=cosb*cosb*(1.-cosa)+cosa;
                rot[0][1]=cosb*sina; rot[1][0]=-rot[0][1];
                rot[0][2]=sinb*cosb*(1.-cosa);rot[2][0]=rot[0][2];
                rot[1][1]=cosa;
                rot[1][2]=sinb*sina; rot[2][1]=-rot[1][2];
/*coordinates xc,yc,zc  error ez   trans formed to ax[3] winstwd=; */
               /* z-coordinate of observer*/
		hf2=100;
                 /* screen --- observer*/
                vertical=(float)(sizey-bottom);
                rx=(float)sizex/stwinsizex;             
                ry=(float)sizey/stwinsizey;
                f1rxry=(float)f1*rx*ry;
                f1f2rxry=(float)(f1+f2)*rx*ry;

		/*dx=plc[a_h].xmax-plc[a_h].xmin;
		  dy=plc[a_h].ymax-plc[a_h].ymin;  10.01.08     
                dx=1.; dy=1.0;
  axes frame --------------------------------------------------
            if(dx > dy){
             wx=0.9*stwinsizex/2;
             wy=wx*dy/dx;
		 }else{*/
	     wx=0.9*stwinsizex/2;
             wy=wx;
             /*wx=wx*dx/dy;}*/
             height=stwinsizey;             
             hwidth=stwinsizex/2;
             hheight=height-bottom;
             cosbwx=cosb*wx;
             sinbwx=sinb*wx;

             xbg=plot.xyzdz[0];
             ybg=plot.xyzdz[2];
             zbg=plot.xyzdz[4];
             cosbwxn=cosb*wx/plot.xyzdz[1];
             sinbwxn=sinb*wx/plot.xyzdz[1];
             wyn=wy/plot.xyzdz[3];
             cosbhtn=cosb*0.75*height/(plot.xyzdz[5]-plot.xyzdz[4]);
             sinbhtn=sinb*0.75*height/(plot.xyzdz[5]-plot.xyzdz[4]);

           jj=2;
           for(npoints=0; npoints<n_data; npoints++){          
	   /*while(p9th[jj-2]+p9th[jj-1]+p9th[jj]!=-3.){
	       while(p9th[jj]!=-1.){*/
	      if(plc[a_h].errorbars){zval=p9th[jj]-zbg-p9th[jj+1];
              }else{zval=p9th[jj]-zbg;}

              axis[0]= cosbwxn*(p9th[jj-2]-xbg)-cosbwx+sinbhtn*zval;
              axis[1]= (p9th[jj-1]-ybg)*wyn-wy;
              axis[2]=-sinbwxn*(p9th[jj-2]-xbg)+sinbwx+cosbhtn*zval;            
              for(i=0;i<3;i++){ax[i]=0.;
                 for(j=0;j<3;j++)ax[i]=ax[i]+rot[j][i]*axis[j];}
              xi=(f1f2rxry-ax[0])/(f1rxry-ax[0]);
	      nval.x=sx+(-ax[1]*(1.-xi)/xi+hwidth)*rx;
              nval.y=sy+(hheight*(cosb+1.)/2.-(hf2-ax[2]*(1.-xi))/xi)*ry;

             if(plc[a_h].errorbars){
              nvalm.x=nval.x;nvalm.y=nval.y;
              zval=p9th[jj]-zbg+p9th[jj+1];
              axis[0]= cosbwxn*(p9th[jj-2]-xbg)-cosbwx+sinbhtn*zval;
              axis[1]= (p9th[jj-1]-ybg)*wyn-wy;
              axis[2]=-sinbwxn*(p9th[jj-2]-xbg)+sinbwx+cosbhtn*zval;           
              for(i=0;i<3;i++){ax[i]=0.;
                 for(j=0;j<3;j++)ax[i]=ax[i]+rot[j][i]*axis[j];}
              xi=(f1f2rxry-ax[0])/(f1rxry-ax[0]);
	      nval.x=sx+(-ax[1]*(1.-xi)/xi+hwidth)*rx;
              nval.y=sy+(hheight*(cosb+1.)/2.-(hf2-ax[2]*(1.-xi))/xi)*ry;
              plot_linevc(handle,nvalm.x,nvalm.y,nval.x,nval.y);
             }else{
	      plot_symbolvc(handle,1,nval.x,nval.y);}
             jj=jj+5;             
	     /*}*/
           }
           if(plc[a_h].residue){
           jj=2;
           zbg=zbg-plot.xyzdz[5]*1.1;
           for(npoints=0; npoints<n_data; npoints++){          
	   /*while(p9th[jj-2]+p9th[jj-1]+p9th[jj]!=-3.){
	       while(p9th[jj]!=-1.){*/
	      if(plc[a_h].errorbars){zval=p9th[jj+2]-zbg-p9th[jj+1];
              }else{zval=p9th[jj+2]-zbg;}

              axis[0]= cosbwxn*(p9th[jj-2]-xbg)-cosbwx+sinbhtn*zval;
              axis[1]= (p9th[jj-1]-ybg)*wyn-wy;
              axis[2]=-sinbwxn*(p9th[jj-2]-xbg)+sinbwx+cosbhtn*zval;            
              for(i=0;i<3;i++){ax[i]=0.;
                 for(j=0;j<3;j++)ax[i]=ax[i]+rot[j][i]*axis[j];}
              xi=(f1f2rxry-ax[0])/(f1rxry-ax[0]);
	      nval.x=sx+(-ax[1]*(1.-xi)/xi+hwidth)*rx;
              nval.y=sy+(hheight*(cosb+1.)/2.-(hf2-ax[2]*(1.-xi))/xi)*ry;

             if(plc[a_h].errorbars){
              nvalm.x=nval.x;nvalm.y=nval.y;
              zval=p9th[jj+2]-zbg+p9th[jj+1];
              axis[0]= cosbwxn*(p9th[jj-2]-xbg)-cosbwx+sinbhtn*zval;
              axis[1]= (p9th[jj-1]-ybg)*wyn-wy;
              axis[2]=-sinbwxn*(p9th[jj-2]-xbg)+sinbwx+cosbhtn*zval;           
              for(i=0;i<3;i++){ax[i]=0.;
                 for(j=0;j<3;j++)ax[i]=ax[i]+rot[j][i]*axis[j];}
              xi=(f1f2rxry-ax[0])/(f1rxry-ax[0]);
	      nval.x=sx+(-ax[1]*(1.-xi)/xi+hwidth)*rx;
              nval.y=sy+(hheight*(cosb+1.)/2.-(hf2-ax[2]*(1.-xi))/xi)*ry;
              plot_linevc(handle,nvalm.x,nvalm.y,nval.x,nval.y);
             }else{
	      plot_symbolvc(handle,1,nval.x,nval.y);}
             jj=jj+5;             
	     /*}*/
           }}
}

static void do_the_plot3d(int handle){
   plot_getdev(handle,&winposx,&winposy,&sizex,&sizey);
   winposx=winposx-2;
   winposy=winposy-24;
/* plot_getdev returns wrong values*/

   size_factor = sizex / 1000;
   if (size_factor < 1.0) size_factor = 1.0;
   /*printf("handle=%d size_factor=%10.3f f1=%d f2=%d sx=%d sy=%d \n",
     handle,size_factor,f1,f2,sx,sy);*/

   plot_setsymsize(handle,size_factor);
   frame = FRAME * size_factor;
   ticlen = TICLEN * size_factor;
   plot_clear(handle);
   plot_setmode(handle,PLOT_MODE_OVER);
   plot_setwin(handle,0,0,sizex,sizey);
   plot_setview(handle,0,0,sizex,sizey);

   if(size!=sizey) plotd_sel_font(handle,sizey);
   size=sizey;
   plot_settextalign(handle, PLOT_LEFTTEXT,PLOT_BOTTOMTEXT); 
/*   plot_writetext(handle, FALSE,
	      sizex*LEFT_FRAME,
	      sizey*TITLE_POSY,plc[a_h].title);*/
   
   plot_setview(handle,sizex*LEFT_FRAME,sizey*BOTTOM_FRAME,
                sizex*RIGHT_FRAME, sizey*TOP_FRAME);
   plot_setclip(handle,TRUE);

   draw_frame3d(handle);
   if(n_data > 0 && plot_data)draw_exp3d(handle);
   if(have_theo && plot_theo) draw_theo3d(handle);
}

void updatetst(){
   int save_plotdata,k3d;
   save_plotdata = plot_data;
   a_h=win.whandle[win.ksphandle-1];
   plot_getdev(a_h,&winposx,&winposy,&sizex,&sizey);
   if (! do_auto_update) return;
   if (a_h == -1) return;
   if (! is_window_device) return;
   plot_data = FALSE; 
   plot_subspectra = TRUE;
   plot_theo = TRUE;
   have_theo = TRUE;
   n_theo = plot.nchanplot[4];
   k3d=win.plotyp[wksp[a_h]];
   if(k3d){draw_theo3d(a_h);
     }else{
     if(plc[a_h].vectplot){
        plot.xyzdz[6]=plc[a_h].yymin;
        plot.xyzdz[7]=plc[a_h].yymax;
        draw_theo2d(a_h,sizex,sizey);
     }
     else mos_plot(a_h);}
   plot_data = save_plotdata;
}

void updatefit(){
   int save_plotsub,save_plotdata,k3d;
   a_h=win.whandle[win.ksphandle-1];
   plot_getdev(a_h,&winposx,&winposy,&sizex,&sizey);
   if (! do_auto_update) return;
   if (a_h == -1) return;
   if (! is_window_device) return;
   save_plotsub = plot_subspectra;
   save_plotdata = plot_data;
   plot_subspectra = FALSE;
   plot_data = FALSE; 
   plot_theo = TRUE;
   have_theo = TRUE;
   n_theo = plot.nchanplot[4];
   k3d=win.plotyp[wksp[a_h]];
   if(k3d){draw_theo3d(a_h);
   }else{
      if(plc[a_h].vectplot){
        plot.xyzdz[6]=plc[a_h].yymin;
        plot.xyzdz[7]=plc[a_h].yymax;
       draw_theo2d(a_h,sizex,sizey);
      }
       else mos_plot(a_h);}
   plot_subspectra = save_plotsub;
   plot_data = save_plotdata;
}

static void write_info(int l, char info[]){
    static char savetext[80] = "";
    int posx, posy;
    plot_settextalign(a_h,PLOT_CENTERTEXT,PLOT_BOTTOMTEXT); 
    posx = sizex*INFO_POSX;    posy = sizey*TOP_FRAME+1.5*frame-(l-1)*sizey*0.04;
    if (savetext[0]!=0) plot_erasetext(a_h,FALSE,posx, posy, savetext);
    plot_writetext(a_h,FALSE,posx,posy,info);
    strcpy(savetext,info);
}


static void zoom(){
   float tmp;
#define exchange(x,y) tmp=x; x=y; y=tmp;
   plc[a_h].last_xmin = plc[a_h].xmin;   plc[a_h].last_xmax = plc[a_h].xmax;
   plc[a_h].last_ymin = plc[a_h].ymin;   plc[a_h].last_ymax = plc[a_h].ymax;

   write_info(1,"Mark first corner");
   plot_getpointer(1,&plc[a_h].xmin,&plc[a_h].ymin);
   write_info(1,"Mark second corner");
   plot_getpointer(1,&plc[a_h].xmax,&plc[a_h].ymax);
   if (plc[a_h].ymax < plc[a_h].ymin) {exchange(plc[a_h].ymax,plc[a_h].ymin);}
   if (plc[a_h].xmax < plc[a_h].xmin) {exchange(plc[a_h].xmax,plc[a_h].xmin);}
   plc[a_h].zoomed = TRUE; 
   win.b_plot[a_h][0]=1; win.r_plot[a_h][0]=plc[a_h].xmin;
                         win.r_plot[a_h][1]=plc[a_h].xmax;
                         win.r_plot[a_h][2]=plc[a_h].ymin;
                         win.r_plot[a_h][3]=plc[a_h].ymax;
#undef exchange
}

/* Swap actual and last data window */
static void unzoom(){
   float tmp;
#define exchange(x,y) tmp=x; x=y; y=tmp;
   if (! plc[a_h].zoomed) return;
   exchange(plc[a_h].xmin, plc[a_h].last_xmin);
   exchange(plc[a_h].xmax, plc[a_h].last_xmax);
   exchange(plc[a_h].ymin, plc[a_h].last_ymin);
   exchange(plc[a_h].ymax, plc[a_h].last_ymax);
#undef exchange
}


static void rescale(){
   plc[a_h].last_xmin = plc[a_h].xmin;   plc[a_h].last_xmax = plc[a_h].xmax;
   plc[a_h].last_ymin = plc[a_h].ymin;   plc[a_h].last_ymax = plc[a_h].ymax;
   if (plc[a_h].scaling == SC_OFF) do_scaling(SC_EXACT);
   else  do_scaling(plc[a_h].scaling);
}

static int is_graphmode = FALSE;

void write_on_win(int k3d){
       /*win.v_plot[a_h][1]=*/
      if(k3d==1){
       win.r_plot[a_h][0]=plc[a_h].alpha0;
       win.r_plot[a_h][1]=plc[a_h].beta0;
       win.r_plot[a_h][2]=plc[a_h].alpha;
       win.r_plot[a_h][3]=plc[a_h].beta;
       win.r_plot[a_h][4]=plc[a_h].sx;
       win.r_plot[a_h][5]=plc[a_h].sy;
       win.r_plot[a_h][6]=plc[a_h].f1;
       win.r_plot[a_h][7]=plc[a_h].f2;
      }else{
       win.v_plot[a_h][0]=plc[a_h].zoomv;
       win.v_plot[a_h][1]=plc[a_h].ixoffset;
       win.v_plot[a_h][2]=plc[a_h].iyoffset;
       win.b_plot[a_h][0]=plc[a_h].vectplot;
       win.b_plot[a_h][1]=plc[a_h].zoomed;
       win.r_plot[a_h][0]=plc[a_h].xmin; 
       win.r_plot[a_h][1]=plc[a_h].xmax;
       win.r_plot[a_h][2]=plc[a_h].ymin; 
       win.r_plot[a_h][3]=plc[a_h].ymax;
      }
}
void write_on_pcl(int k3d){
       /*win.v_plot[a_h][1]=*/
      if(k3d==1){
       if( win.r_plot[a_h][7]==1000){
       plc[a_h].alpha0=win.r_plot[a_h][0];
       plc[a_h].beta0 =win.r_plot[a_h][1];
       plc[a_h].alpha =win.r_plot[a_h][2];
       plc[a_h].beta  =win.r_plot[a_h][3];
       plc[a_h].sx    =win.r_plot[a_h][4];
       plc[a_h].sy    =win.r_plot[a_h][5];
       plc[a_h].f1    =win.r_plot[a_h][6];
       plc[a_h].f2    =win.r_plot[a_h][7];} 
      }else{
       plc[a_h].zoomv=win.v_plot[a_h][0];
       if(plc[a_h].zoomv!= 0){
       plc[a_h].ixoffset=win.v_plot[a_h][1];
       plc[a_h].iyoffset=win.v_plot[a_h][2];}
       plc[a_h].vectplot=win.b_plot[a_h][0];
       plc[a_h].zoomed=win.b_plot[a_h][1];
       if(plc[a_h].zoomed){
       plc[a_h].xmin  =win.r_plot[a_h][0]; 
       plc[a_h].xmax  =win.r_plot[a_h][1];
       plc[a_h].ymin  =win.r_plot[a_h][2]; 
       plc[a_h].ymax  =win.r_plot[a_h][3];}
      }
}

int plot_the_buffers_noask(){
  float x, y, dalpha,dbeta;
  int trm ,lhandle,f2,irx11;
  char out[40],chin[132];
  int lfirst,ifirst, last, ldf, kstayin, lasttrm, follow, k3d, ix,iy; 
  time_t t1,t2;
   zoomf=plc[a_h].zoomv;
   plc[a_h].residue=have_residue;        /* 05.10.10 */

  follow=0;
  lfirst=0;
  irx11=0;
  last=-ldf;
  kstayin=1;

  f2=1000; lasttrm=1;
  time(&t2); 
  do{
    lfirst=lfirst+1;
    lhandle=a_h;
    k3d=win.plotyp[wksp[a_h]];

    /* printf("plot_the_buffers_noask k3d=%d lhandle=%d\n",k3d,lhandle);*/

    if (!is_graphmode) plot_graphmode();
    is_graphmode = TRUE;
    time(&t1);
    if(trm==255)ldf=4;
    if(((int)t1-t2 >0 || lfirst > last+ldf) && trm!=258){
	last=lfirst;
        plot_setfgbyname(a_h,plot.fgcolor);
/*printf("  lfirst-last=%d  term=%d time=%d\n",lfirst-last,trm,(int)t1-t2);*/
     if(k3d){  do_the_plot3d(a_h);}
      else {if(plc[a_h].vectplot)do_the_plot2d(a_h); 
                            else do_the_plot(a_h);}   
     if(lfirst == 1)plotd_focus_X11(a_h);
    }
     time(&t2);

    if (plot_auto){plot_auto=FALSE;  return '*';}
    plot_setfgbyname(a_h,plot.fgcolor);
    if(irx11==0) write_info(1,"press H to get help");
    /*if((k==FALSE) return 'M';*/
    if(win.hold == 0) return 'M';
    if(k3d==0)kstayin=1;

     win.posx[win.ksphandle-1]=winposx;    
     win.posy[win.ksphandle-1]=winposy;
     win.sizex[win.ksphandle-1]=sizex;
     win.sizey[win.ksphandle-1]=sizey;

    if(irx11 == 1){if(numplot==0)file_close(a_h);
                   plotd_func_x11(handles[a_h]); irx11=0;iframe=3;
                   if(numplot==0)numpl=1;else numpl=-1; notitle=0;}
     do{   ldf=0;  
/*printf("k3d=%d  lfirst=%d  term=%d kstayin=%d \n",k3d,lfirst,trm,kstayin);*/
	if(k3d){trm = plot_getpointer3d(0,&ix,&iy); if(trm==257)ldf=3; 
         	/*printf("3dplot  trm=%d x=%d  y=%d \n",trm,ix,iy);*/
        	plc[a_h].vectplot=1;
                if(lasttrm < 0 && trm == 257){kstayin=1;lasttrm=1;}
                if(follow && trm == 257){
                 plc[a_h].alpha=(float)ix/sizex*3.14;
        	 plc[a_h].beta=-(float)iy/sizey*3.14/2.;
		 if(ifirst){dalpha=plc[a_h].alpha0-plc[a_h].alpha;
                            dbeta=plc[a_h].beta0-plc[a_h].beta; ifirst=0;}
                 plc[a_h].alpha=plc[a_h].alpha+dalpha; 
                 plc[a_h].beta=plc[a_h].beta+dbeta;
		 /*printf("3dplotfol k=%d alpha=%8.3f  beta=%8.3f \n",
                                                   kstayin,alpha,beta);*/
                 break;}

                if(trm < 0/*Arrow*/){kstayin=0;lasttrm=-1;}
		if(trm == -102/*rightArrow*/){plc[a_h].sx=plc[a_h].sx+10;break;}
 		if(trm == -100/*leftArrow*/) {plc[a_h].sx=plc[a_h].sx-10;break;}
		if(trm == -98/*upArrow*/)    {plc[a_h].sy=plc[a_h].sy-10;break;}
 		if(trm == -104/*downArrow*/) {plc[a_h].sy=plc[a_h].sy+10;break;}
                if(trm == 241/*left ButtonPress*/)
                                 {follow=1;kstayin=0;ifirst=1;break;}
                if(trm == 258/*ButtonRelease*/){follow=0; kstayin=1;
                                       plc[a_h].alpha0=plc[a_h].alpha;
                                       plc[a_h].beta0=plc[a_h].beta;break;}
        }else{
	 if(plc[a_h].vectplot){ trm =plot_getpointer2d(0,&x,&y);
                if(trm < 0/*Arrow*/){kstayin=0;lasttrm=-1;}
		if(trm == -102/*rightArrow*/){
                plc[a_h].ixoffset=plc[a_h].ixoffset+10;break;}
 		if(trm == -100/*leftArrow*/) {
                 plc[a_h].ixoffset=plc[a_h].ixoffset-10;break;}
		if(trm == -98/*upArrow*/){
                 plc[a_h].iyoffset=plc[a_h].iyoffset+10;break;}
 		if(trm == -104/*downArrow*/){
                plc[a_h].iyoffset=plc[a_h].iyoffset-10;break;}
	  }else{trm = plot_getpointer(0,&x,&y);}
          if(trm == 241){ follow=1; kstayin=1;break;}
          if(trm == 258){ follow=0; kstayin=0;break;}
        /*printf("plot_the_buffers_noask x=%f  y=%f \n",x,y);*/
       }
       
	/*printf("plot_the_buffers_noask trm=%d \n",trm);
      printf("plot_the_buffers_noask lhandle=%d a_h=%d \n",lhandle,a_h);*/
      if(lhandle!=a_h) return 'C';
      /*trm = toupper(trm);*/

      if (trm==PLOT_TRM_MB_RIGHT) return '*';
      if (trm=='q') return '*';
      if (trm=='*') return '*';
      if (trm=='P'){if(numplot == 0){
            write_info(1,"second character:  p=portrait, l=landscape, a=allplo format  or");
            write_info(2,"second character:  2,3... plots on A4 (portrait)               ");
            write_info(3,"recover plot by left mouse click");}else{
            write_info(1,"second character: p=portrait)");
            write_info(2,"recover plot by left mouse click");}
                   trm =plot_getpointer2d(0,&x,&y); 
            if(trm > 48 && trm < 58 && numpl == 1)numplot=trm-48;
            if(trm == 'a'){notitle=1;
	                   plotd_func_dsp(handles[a_h],plc[a_h].vectplot,trm);}
            else plotd_func_ps(handles[a_h],plc[a_h].vectplot,trm,numpl,numplot);
               irx11=1;iframe=7;if(numplot>0)numplot=numplot-1;
               break;}

      if (trm==PLOT_TRM_REPLOT) break; /* 255*/
      if (trm==PLOT_TRM_ALREADY_ACTIVE) return trm;/* 256*/
      if (trm=='r') {if(k3d){plc[a_h].f1=500;plc[a_h].f2=1000;
                             plc[a_h].sx=0;plc[a_h].sy=0;
                             plc[a_h].alpha=plc[a_h].alpha0;
                             plc[a_h].beta=plc[a_h].beta0;
                    }else{
                     plc[a_h].zoomed =0;rescale();
                     plc[a_h].zoomv=0;
                     plc[a_h].ixoffset=0; plc[a_h].iyoffset=0;} break;}
      if (trm=='h'||trm=='H') {if(k3d)sprintf(chin,"%s","effi/xmosplo/3d $");
                                 else sprintf(chin,"%s","effi/xmosplo/2d $");
                                 help(chin);break;}
      if (trm=='z') {if(k3d){plc[a_h].f1=plc[a_h].f1-100;
                     if(plc[a_h].f1==-plc[a_h].f2)plc[a_h].f1=-plc[a_h].f2+200;
                    }else{if(plc[a_h].vectplot) plc[a_h].zoomv=plc[a_h].zoomv+1;
			  else zoom(); } break;}
      if (trm=='Z') {if(k3d){plc[a_h].f1=plc[a_h].f1+100;
                    }else{if(plc[a_h].vectplot)plc[a_h].zoomv=plc[a_h].zoomv-1;
			  else unzoom();}; break;}
      if (trm=='d') {plot_data = (! plot_data); break;}
      if (trm=='t') {plot_theo = (! plot_theo); break;}
      if (trm=='e') {plc[a_h].errorbars= (! plc[a_h].errorbars); break;}
      if (trm=='E') {plc[a_h].residue= (! plc[a_h].residue); break;}
      if (trm=='v') {plc[a_h].vectplot= (! plc[a_h].vectplot); 
                     /*printf("vectplot=%d \n", plc[a_h].vectplot);*/
                     kstayin=0;break;}
      if (k3d==0){
       if (trm=='a') {if(plc[a_h].scaling==SC_OFF){plc[a_h].scaling=SC_EXACT;}
                     else{plc[a_h].scaling=SC_OFF;};break;}
      if (trm=='s'){plot_subspectra=(! plot_subspectra);
                    is_graphmode=FALSE; break;}
      if (trm=='1'){ pl_subsp[0]=(! pl_subsp[0]);is_graphmode=FALSE; break;}
      if (trm=='2'){ pl_subsp[1]=(! pl_subsp[1]);is_graphmode=FALSE; break;}
      if (trm=='3'){ pl_subsp[2]=(! pl_subsp[2]);is_graphmode=FALSE; break;}
      if (trm=='4'){ pl_subsp[3]=(! pl_subsp[3]);is_graphmode=FALSE; break;}
      if (trm=='5'){ pl_subsp[4]=(! pl_subsp[4]);is_graphmode=FALSE; break;}
      if (trm=='6'){ pl_subsp[5]=(! pl_subsp[5]);is_graphmode=FALSE; break;}
      if (trm=='7'){ pl_subsp[6]=(! pl_subsp[6]);is_graphmode=FALSE; break;}
      if (trm=='8'){ pl_subsp[7]=(! pl_subsp[7]);is_graphmode=FALSE; break;}

      sprintf(out,"%10.4f  %15.5f",x,y);
      if (is_window_device && trm == 243) printf("%s\n",out);
      if(follow) write_info(1,out);
      }else{
       if (trm=='1'){plc[a_h].netr =! plc[a_h].netr;break;}
       if (trm=='2'){plc[a_h].netl =! plc[a_h].netl;break;}
      }       
    }while(kstayin);

  }while((trm!='*') && (trm!='M'));
  if(numplot>0)file_close(a_h);
  return trm;
}

static void do_plot_the_buffers()
{
   int trm,k3d;

   if (a_h < 0) return;
   k3d=win.plotyp[wksp[a_h]];
   if(k3d == 0){
    /*printf("SC_OFF=%d  plc[a_h].scaling=%d \n", SC_OFF,plc[a_h].scaling);*/
    if (plc[a_h].scaling != SC_OFF) do_scaling(plc[a_h].scaling);
    if (plc[a_h].xmin >= plc[a_h].xmax) {
     printf("** error**  xmosplo:  xmin= %10.2f xmax= %10.2f \n",plc[a_h].xmin,plc[a_h].xmax);
     win.ksphandle=-2;
     return;}
   }
   do{
     plc[a_h].yymin=plot.xyzdz[6];
     plc[a_h].yymax=plot.xyzdz[7];
     trm = plot_the_buffers_noask();
     if (trm==PLOT_TRM_ALREADY_ACTIVE) return;
     is_graphmode = FALSE;
     plot_textmode();
       /*      printf("do_plot_the_buffers() trm  %d M %d\n",trm,'M');*/
/*     win.posx[win.ksphandle-1]=winposx;    
     win.posy[win.ksphandle-1]=winposy;
     win.sizex[win.ksphandle-1]=sizex;
     win.sizey[win.ksphandle-1]=sizey;*/
/*     printf("ksphandle=%i  sizex=%i sizey=%i winposx=%i  winposy=%i\n",\
       win.ksphandle,sizex,sizey,winposx,winposy);*/
     if(trm=='*') win.ksphandle=-1;
     if(trm=='M'){ win.ksphandle=0;trm='*';}
     if(trm=='C'){win.ksphandle=wksp[a_h]+1;trm='*';}
   }while(trm!='*');
}

static void copy_title(){
   char *tmp;
   int i, ii;
   tmp = win.wdescript;
   for (i=0; i<79; i++){
       ii = i + (i/2)*2;
       plc[a_h].wdescript[i] = tmp[i]; 
   };plc[a_h].wdescript[79] = 0;
   tmp = plot.title;
   for (i=0; i<79; i++){
       ii = i + (i/2)*2;
       plc[a_h].title[i] = tmp[i]; 
   };plc[a_h].title[79] = 0;

   tmp= plot.axisformat[0];
   for (i=0; i<16; i++){if(tmp[i]==' '){plc[a_h].xformat[i] = 0;}
   else{plc[a_h].xformat[i] = tmp[i];}}
   plc[a_h].xformat[16]=0;

   tmp= plot.axisformat[0];
   for (i=0; i<16; i++){if(tmp[i+16]==' '){plc[a_h].yformat[i] = 0;}
   else{plc[a_h].yformat[i] = tmp[i+16];}}
   plc[a_h].yformat[16]=0;

   for (i=0; i<24; i++) plc[a_h].xlabel[i] = 0;
   tmp= plot.axes[0];   ii=0;
   for (i=23; i>1; i--){if(tmp[i] != ' '){ii=i+1;break;};}
   for (i=0; i<ii; i++) plc[a_h].xlabel[i] = tmp[i];

   for (i=0; i<24; i++) plc[a_h].ylabel[i] ='\0';
   tmp= plot.axes[0];   ii=0;
   for (i=23; i>1; i--){if(tmp[i+24] != ' '){ii=i+1;break;};}
   for (i=0; i<ii; i++) plc[a_h].ylabel[i] = tmp[i+24];
}

void xmosplo(int *n1, int *iauto){
  int handle,i,ii,k3d;
  char *tmp;
  if(*n1 == 2){ updatetst();return;}
  if(*n1 == 3){ updatefit();return;}
     winsizex=win.sizex[win.ksphandle-1];
     winsizey=win.sizey[win.ksphandle-1];
     winposx=win.posx[win.ksphandle-1];
     winposy=win.posy[win.ksphandle-1];
     /*printf(" handle=%i winsizex=%i winsizey=%i winposx=%i  winposy=%i\n", \
       win.ksphandle,winsizex,winsizey,winposx,winposy);*/
       k3d=*iauto/2;
     /*k=TRUE;
       if(win.hold == 0) k=FALSE;*/
   if (win.whandle[win.ksphandle-1]<0){
       handle=0;
       tmp = win.wdescript;
       for (i=0; i<79; i++){ii = i + (i/2)*2;
       wdescription[i] = tmp[i];};wdescription[79] = 0;
       init_plot(handle,"x",plot.bgcolor);
       if(a_h <0) {win.ksphandle=-1; return;}
       do_auto_update = TRUE; 
       wptr.intptr[a_h]=(long)handles[a_h];
       win.whandle[win.ksphandle-1]=a_h;
   }else{   
    for (i=0; i<79; i++){wdescription[i] = plc[a_h].wdescript[i];}
   }
   win.plotyp[win.ksphandle-1]=k3d;
   a_h=win.whandle[win.ksphandle-1]; 
   wksp[a_h]=win.ksphandle-1;
   i=0;
   for(handle=0; handle<MAXHANDLES; handle++){ 
     if(win.whandle[handle]>=0){i=i+1;}
   }
   mhandle=i;
   /*printf("k3d 0=%d 1=%d 2=%d\n",win.plotyp[0],win.plotyp[1],win.plotyp[2]);*/
   copy_title();
   have_theo = (*n1 == 1);
   plot_theo = have_theo;
   n_data = plot.nchanplot[0];
   have_data=(n_data > 1);
   plot_data = have_data;
   have_residue=(plot.nchanplot[3] > 1); 

   if(k3d){ 
    n_theo = plot.nchanplot[5];
    n_sub = 0;
   }else{
    n_theo = plot.nchanplot[4];
    n_sub =  plot.n_subspectra;
   }; 
   have_subspectra=(n_sub>1);
   if(have_subspectra){
    plot_subspectra = TRUE;
    for (i=0; i<n_sub; i++){pl_subsp[i]=TRUE;};}

   plot_auto=(*iauto == 0 || *iauto == 2);
/* pointers p** on different entries of plot.array[] */
   pxd=&plot.array[0];
   pyd=&plot.array[MAX8];
   pyderr=&plot.array[MAX8*2];
   pyres=&plot.array[MAX8*3];
   pxt=&plot.array[MAX8*4];
   pyt=&plot.array[MAX8*5];
   for(i=0; i<8; i++){
       psub[i]=&plot.array[MAX8*(6+i)];}  
   p6th=&plot.array[MAX8*6];
   p9th=&plot.array[MAX8*9];

   pfeder=&plot.ifeder[0];

   write_on_pcl(k3d);
   do_plot_the_buffers();
   write_on_win(k3d);
}







